﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace fblock.manager
{
    public abstract class FBlockAccessorBase : DefaultFBlockAccessorEvents, IFBlockAccessor
    {
        IReportSink reporter = null;
        bool showUI = true;
        bool connected = false;
        AutoResetEvent waitDisposed = new AutoResetEvent(false);

        public FBlockAccessorBase()
            : base()
        {
            // Consume those events to handle connect/disconnect internally.
            base.Connected += new EventHandler<EventArgs>(FBlockAccessorBase_Connected);
            base.Disconnected += new EventHandler<EventArgs>(FBlockAccessorBase_Disconnected);
        }

        #region FBlockAccessorBase abstract member declaration

        public abstract Form GetForm();
        public abstract string GetName();
        public abstract string GetCategory();
        public abstract string GetDescription();
        public abstract string GetVersion();
        public abstract string GetCatalog();
        public abstract int GetFBlockID();
        public abstract int GetInstanceID();
        public abstract bool RunInitialize();
        public abstract bool RunCleanup();
        public abstract bool RunConnect();
        public abstract bool RunDisconnect();
        public abstract bool RunLoadConfiguration();
        public abstract bool RunSaveConfiguration();

        #endregion // FBlockAccessorBase abstract member declaration

        #region IFBlockAccessor Members

        public string Name
        {
            get { return this.GetName(); }
        }

        public string Category
        {
            get { return this.GetCategory(); }
        }

        public string Description
        {
            get { return this.GetDescription(); }
        }

        public string Version
        {
            get { return this.GetVersion(); }
        }

        public string Catalog
        {
            get { return this.GetCatalog(); }
        }

        public int FBlockID
        {
            get { return this.GetFBlockID(); }
        }

        public int InstanceID
        {
            get { return this.GetInstanceID(); }
        }

        public bool IsConnected
        {
            get { return this.connected; }
        }

        public IReportSink ReportSink
        {
            get { return this.reporter; }
        }

        public bool ShowUI
        {
            get
            {
                return this.showUI;
            }
            set
            {
                if (this.showUI != value)
                {
                    this.showUI = value;

                    if (this.GetForm() != null)
                    {
                        if (this.GetForm().Visible)
                        {
                            if (this.showUI)
                            {
                                this.GetForm().WindowState = FormWindowState.Normal;

                                this.FireShowUIChangedEvent(
                                    this.showUI,
                                    ShowUIChangedEventArgs.ShowState.Shown);
                            }
                            else
                            {
                                this.GetForm().Hide();

                                this.FireShowUIChangedEvent(
                                    this.showUI,
                                    ShowUIChangedEventArgs.ShowState.Hidden);
                            }
                        }
                        else
                        {
                            if (this.showUI)
                            {
                                this.GetForm().Show();
                                
                                this.FireShowUIChangedEvent(
                                    this.showUI,
                                    ShowUIChangedEventArgs.ShowState.Shown);
                            }
                            else
                            {
                                this.GetForm().Hide();
                               
                                this.FireShowUIChangedEvent(
                                    this.showUI,
                                    ShowUIChangedEventArgs.ShowState.Hidden);
                            }
                        }
                    }
                }

                // Bring form to front.
                if (this.showUI) { this.GetForm().BringToFront(); }
            }
        }

        public Form MainForm
        {
            get { return this.GetForm(); }
        }

        public bool Initialize(IReportSink reporter)
        {
            bool success = true; // Do not fail if main form is not available.
            
            // Save given report sink.
            this.reporter = reporter;

            if (this.GetForm() != null)
            {
                this.GetForm().Text = this.Name + " " + this.Version;

                if (!this.WindowSubClassing(this.GetForm()) && this.reporter != null)
                {
                    this.reporter.Error("Sub-classing the FBlock's form failed.", "FBlockAccessorBase", "Initialize");
                }

                this.GetForm().FormClosing += new FormClosingEventHandler(MainForm_FormClosing);
                this.GetForm().Move += new EventHandler(MainForm_Move);

                // Initialize derived class.
                if (!this.RunInitialize())
                {
                    success = false;

                    if (this.reporter != null)
                    {
                        this.reporter.Error("Initialization of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Initialize");
                    }
                }

                // Load configuration of derived class.
                if (!this.LoadConfiguration())
                {
                    success = false;
                }

                // Show form of derived class depending on current state.
                if (this.ShowUI)
                {
                    this.GetForm().WindowState = FormWindowState.Normal;
                    this.GetForm().Show();
                }
                else
                {
                    this.GetForm().Hide();
                }
            }

            return success;
        }

        public bool Cleanup()
        {
            bool success = true; // Do not fail if main form is not available.

            if (this.GetForm() != null)
            {
                // Save configuration of derived class.
                if (!this.SaveConfiguration())
                {
                    success = false;
                }

                // Cleanup derived class.
                if (!this.RunCleanup())
                {
                    success = false;

                    if (this.reporter != null)
                    {
                        this.reporter.Error("Cleanup of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Cleanup");
                    }
                }

                this.GetForm().Move -= new EventHandler(MainForm_Move);
                this.GetForm().FormClosing -= new FormClosingEventHandler(MainForm_FormClosing);
                this.GetForm().Disposed += new EventHandler(MainForm_Disposed);

                this.GetForm().Close();

                WaitHandle.WaitAll(new WaitHandle[] { this.waitDisposed });
            }

            return success;
        }

        public bool Connect()
        {
            bool success = this.RunConnect();

            if (!success && this.reporter != null)
            {
                this.reporter.Error("Connection of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Connection");
            }

            return success;
        }

        public bool Disconnect()
        {
            bool success = this.RunDisconnect();
            
            if (!success && this.reporter != null)
            {
                this.reporter.Error("Disconnection of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Connection");
            }

            return success;
        }

        public bool LoadConfiguration()
        {
            bool success = this.RunLoadConfiguration();
            
            if (!success && this.reporter != null)
            {
                this.reporter.Error("Loading configuration of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Configuration");
            }
            
            return success;
        }

        public bool SaveConfiguration()
        {
            bool success = this.RunSaveConfiguration();
            
            if (!success && this.reporter != null)
            {
                this.reporter.Error("Saving configuration of " + this.Category + " \"" + this.Name + "\" failed.", "FBlockAccessorBase", "Configuration");
            }
            
            return success;
        }

        #endregion // IFBlockAccessor Members

        #region Interface event handling members

        private void FBlockAccessorBase_Connected(object sender, EventArgs args)
        {
            this.connected = true;
        }

        private void FBlockAccessorBase_Disconnected(object sender, EventArgs args)
        {
            this.connected = false;
        }

        #endregion //  Interface event handling members

        #region Windows event handling members

        private void MainForm_FormClosing(object sender, FormClosingEventArgs args)
        {
            if (this.GetForm() != null)
            {
                args.Cancel = true;

                this.GetForm().Hide();

                this.showUI = false;
                
                this.FireShowUIChangedEvent(
                    this.showUI,
                    ShowUIChangedEventArgs.ShowState.Hidden);
            }
        }

        private void MainForm_Move(object sender, EventArgs args)
        {
            this.FireLocationChangedEvent();
        }

        private void MainForm_Disposed(object sender, EventArgs args)
        {
            this.waitDisposed.Set();
        }
        
        #endregion // Windows event handling members

        #region Window sub-classing code

        #region User32 import section

        [DllImport("user32.dll")]
        private static extern IntPtr GetWindowLong(IntPtr handle, int index);

        [DllImport("user32.dll")]
        private static extern int SetWindowLong(IntPtr handle, int nIndex, IntPtr value);

        [DllImport("user32.dll")]
        private static extern IntPtr CallWindowProc(IntPtr previous, IntPtr handle, uint message, IntPtr wParam, IntPtr lParam);

        #endregion // User32 import section

        private IntPtr oldProcedure = IntPtr.Zero;
        private WndProcDelegate newProcedure = null;
        
        private delegate IntPtr WndProcDelegate(IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam);

        private IntPtr WindowHandler(IntPtr handle, uint message, IntPtr wParam, IntPtr lParam)
        {
            const uint WM_SYSCOMMAND = 0x0112;
            const int SC_MINIMIZE = 0xF020;
            const int SC_MAXIMIZE = 0xF030;
            const int SC_RESTORE = 0xF120;

            if (message == WM_SYSCOMMAND)
            {
                int id = wParam.ToInt32() & 0xFFF0;

                if (id == SC_MINIMIZE)
                {
                    this.showUI = false;
                    this.FireShowUIChangedEvent(
                        this.showUI,
                        ShowUIChangedEventArgs.ShowState.Minimized);
                }
                else if (id == SC_MAXIMIZE)
                {
                    this.showUI = true;
                    this.FireShowUIChangedEvent(
                        this.showUI, 
                        ShowUIChangedEventArgs.ShowState.Maximized);
                }
                else if (id == SC_RESTORE)
                {
                    this.showUI = true;
                    this.FireShowUIChangedEvent(
                        this.showUI, 
                        ShowUIChangedEventArgs.ShowState.Shown);
                }
            }

            return CallWindowProc(oldProcedure, handle, message, wParam, lParam);
        }

        private bool WindowSubClassing(Form window)
        {
            const int GWL_WNDPROC = (-4);

            newProcedure = new WndProcDelegate(WindowHandler);
            oldProcedure = GetWindowLong(window.Handle, GWL_WNDPROC);

            int success = SetWindowLong(window.Handle, GWL_WNDPROC, Marshal.GetFunctionPointerForDelegate(newProcedure));
            return (success != 0);
        }

        #endregion // Window sub-classing code
    }
}

// EOF