﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;

namespace fblock.manager
{
    public class ApplicationSettings
    {
        private static ApplicationSettings theInstance = null; // Singleton instance.

        private string lastProjectPath;
        private string lastProjectFile;
        private string lastImportFilePath;
        private string lastImportFolderPath;
        private List<string> mruProjects;
        private int mruMaxProjects;
        private Point mainFormLocation;
        private Size mainFormDimension;
        private FormWindowState mainFormWindowState;
        private bool autoOpenLastProject;
        private bool autoFBlockConnection;
        private bool useFBlockIDNameLabel;
        private bool reportShowErrors;
        private bool reportShowWarnings;
        private bool reportShowMessages;
        private int mainSplitDistance;
        private int treeSplitDistance;
        private bool autoScroll;
        private bool singleton;
        private DoubleClickAction doubleClickAction;
        private Color infoPaneForeground;
        private Color infoPaneBackground;
        private Color treeViewForeground;
        private Color treeViewBackground;
        private Color reportSinkForeground;
        private Color reportSinkBackground;
        private LoggingSettings logging;
        private ulong reportMaxEntries; // Zero means unlimited!

        private bool loaded = false;
        private bool serializing = false;

        private ApplicationSettings()
        {
            this.InitializeDefaults();
        }

        public static ApplicationSettings Instance
        {
            get
            {
                if (theInstance == null)
                {
                    theInstance = new ApplicationSettings();
                }
                return theInstance;
            }
        }

        public bool Loaded
        {
            get { return this.loaded; }
        }

        public static ApplicationSettings CreateCopy()
        {
            ApplicationSettings result = new ApplicationSettings();

            if (theInstance != null)
            {
                result.lastProjectPath = theInstance.lastProjectPath;
                result.lastProjectFile = theInstance.lastProjectFile;
                result.lastImportFilePath = theInstance.lastImportFilePath;
                result.lastImportFolderPath = theInstance.lastImportFolderPath;
                result.mruProjects.AddRange(theInstance.mruProjects.ToArray());
                result.mruMaxProjects = theInstance.mruMaxProjects;
                result.mainFormLocation = theInstance.mainFormLocation;
                result.mainFormDimension = theInstance.mainFormDimension;
                result.mainFormWindowState = theInstance.mainFormWindowState;
                result.autoOpenLastProject = theInstance.autoOpenLastProject;
                result.autoFBlockConnection = theInstance.autoFBlockConnection;
                result.useFBlockIDNameLabel = theInstance.useFBlockIDNameLabel;
                result.reportShowErrors = theInstance.reportShowErrors;
                result.reportShowWarnings = theInstance.reportShowWarnings;
                result.reportShowMessages = theInstance.reportShowMessages;
                result.mainSplitDistance = theInstance.mainSplitDistance;
                result.treeSplitDistance = theInstance.treeSplitDistance;
                result.autoScroll = theInstance.autoScroll;
                result.singleton = theInstance.singleton;
                result.doubleClickAction = theInstance.doubleClickAction;
                result.infoPaneForeground = theInstance.infoPaneForeground;
                result.infoPaneBackground = theInstance.infoPaneBackground;
                result.treeViewForeground = theInstance.treeViewForeground;
                result.treeViewBackground = theInstance.treeViewBackground;
                result.reportSinkForeground = theInstance.reportSinkForeground;
                result.reportSinkBackground = theInstance.reportSinkBackground;
                result.logging.Copy(theInstance.logging);
                result.reportMaxEntries = theInstance.reportMaxEntries;
            }

            return result;
        }

        public static void UseFromCopy(ApplicationSettings copy)
        {
            if (theInstance != null && copy != null)
            {
                theInstance.lastProjectPath = copy.lastProjectPath;
                theInstance.lastProjectFile = copy.lastProjectFile;
                theInstance.lastImportFilePath = copy.lastImportFilePath;
                theInstance.lastImportFolderPath = copy.lastImportFolderPath;
                theInstance.mruMaxProjects = copy.mruMaxProjects;

                // Do it in this way to ensure list length limitations!
                theInstance.mruProjects.Clear();
                if (copy.mruMaxProjects > 0)
                {
                    int limit = (copy.mruProjects.Count > copy.mruMaxProjects) ? copy.mruMaxProjects : copy.mruProjects.Count;
                    string[] temp = new string[limit];
                    copy.mruProjects.CopyTo(0, temp, 0, limit);
                    theInstance.mruProjects.AddRange(temp);
                }
                else
                {
                    theInstance.mruProjects.AddRange(copy.mruProjects.ToArray());
                }

                theInstance.mainFormLocation = copy.mainFormLocation;
                theInstance.mainFormDimension = copy.mainFormDimension;
                theInstance.mainFormWindowState = copy.mainFormWindowState;
                theInstance.autoOpenLastProject = copy.autoOpenLastProject;
                theInstance.autoFBlockConnection = copy.autoFBlockConnection;
                theInstance.useFBlockIDNameLabel = copy.useFBlockIDNameLabel;
                theInstance.reportShowErrors = copy.reportShowErrors;
                theInstance.reportShowWarnings = copy.reportShowWarnings;
                theInstance.reportShowMessages = copy.reportShowMessages;
                theInstance.mainSplitDistance = copy.mainSplitDistance;
                theInstance.treeSplitDistance = copy.treeSplitDistance;
                theInstance.autoScroll = copy.autoScroll;
                theInstance.singleton = copy.singleton;
                theInstance.doubleClickAction = copy.doubleClickAction;
                theInstance.infoPaneForeground = copy.infoPaneForeground;
                theInstance.infoPaneBackground = copy.infoPaneBackground;
                theInstance.treeViewForeground = copy.treeViewForeground;
                theInstance.treeViewBackground = copy.treeViewBackground;
                theInstance.reportSinkForeground = copy.reportSinkForeground;
                theInstance.reportSinkBackground = copy.reportSinkBackground;
                theInstance.logging.Copy(copy.logging);
                theInstance.reportMaxEntries = copy.reportMaxEntries;
            }
        }

        [XmlIgnore]
        public string LastProjectPath
        {
            get
            {
                if (this.serializing)
                {
                    return this.lastProjectPath;
                }
                else
                {
                    return ApplicationSettings.AbsolutPath(this.lastProjectPath);
                }
            }

            set { this.lastProjectPath = ApplicationSettings.RelativePath(value); }
        }

        [XmlIgnore]
        public string LastProjectFile
        {
            get { return this.lastProjectFile; }
            set { this.lastProjectFile = value; }
        }

        public string LastProject
        {
            get
            {
                return Path.Combine(this.LastProjectPath, this.LastProjectFile);
            }

            set
            {
                if (String.IsNullOrEmpty(value))
                {
                    this.LastProjectPath = "";
                    this.LastProjectFile = "";
                }
                else
                {
                    this.LastProjectPath = Path.GetDirectoryName(value);
                    this.LastProjectFile = Path.GetFileName(value);
                }
            }
        }

        public string LastFileImport
        {
            get
            {
                if (this.serializing)
                {
                    return this.lastImportFilePath;
                }
                else
                {
                    return ApplicationSettings.AbsolutPath(this.lastImportFilePath);
                }
            }

            set { this.lastImportFilePath = ApplicationSettings.RelativePath(value); }
        }

        public string LastFolderImport
        {
            get
            {
                if (this.serializing)
                {
                    return this.lastImportFolderPath;
                }
                else
                {
                    return ApplicationSettings.AbsolutPath(this.lastImportFolderPath);
                }
            }

            set { this.lastImportFolderPath = ApplicationSettings.RelativePath(value); }
        }

        public string[] MRUProjects
        {
            get
            {
                if (this.serializing)
                {
                    // Use a temporary copy of current MRU list to serialize it!
                    string[] result = new string[this.mruProjects.Count];
                    for (int index = 0; index < this.mruProjects.Count; index++)
                    {
                        result[index] = ApplicationSettings.RelativePath(this.mruProjects[index]);
                    }
                    return result;
                }
                else
                {
                    return this.mruProjects.ToArray();
                }
            }

            set
            {
                // Prepare current MRU list from given path array
                this.mruProjects.Clear();
                if (value != null)
                {
                    foreach (string current in value)
                    {
                        this.mruProjects.Add(ApplicationSettings.AbsolutPath(current));
                    }
                }
            }
        }

        public int MRUMaxProjects
        {
            get { return this.mruMaxProjects; }
            set { this.mruMaxProjects = value; }
        }

        public Point MainFormLocation
        {
            get { return this.mainFormLocation; }
            set { this.mainFormLocation = value; }
        }

        public Size MainFormDimension
        {
            get { return this.mainFormDimension; }
            set { this.mainFormDimension = value; }
        }

        public FormWindowState MainFormWindowState
        {
            get { return this.mainFormWindowState; }
            set { this.mainFormWindowState = value; }
        }

        public bool AutoOpenLastProject
        {
            get { return this.autoOpenLastProject; }
            set { this.autoOpenLastProject = value; }
        }

        public bool AutoFBlockConnection
        {
            get { return this.autoFBlockConnection; }
            set { this.autoFBlockConnection = value; }
        }

        public bool UseFBlockIDNameLabel
        {
            get { return this.useFBlockIDNameLabel; }
            set { this.useFBlockIDNameLabel = value; }
        }

        public bool ReportShowErrors
        {
            get { return this.reportShowErrors; }
            set { this.reportShowErrors = value; }
        }

        public bool ReportShowWarnings
        {
            get { return this.reportShowWarnings; }
            set { this.reportShowWarnings = value; }
        }

        public bool ReportShowMessages
        {
            get { return this.reportShowMessages; }
            set { this.reportShowMessages = value; }
        }

        public int MainSplitDistance
        {
            get { return this.mainSplitDistance; }
            set { this.mainSplitDistance = value; }
        }

        public int TreeSplitDistance
        {
            get { return this.treeSplitDistance; }
            set { this.treeSplitDistance = value; }
        }

        public bool AutoScroll
        {
            get { return this.autoScroll; }
            set { this.autoScroll = value; }
        }

        public bool Singleton
        {
            get { return this.singleton; }
            set { this.singleton = value; }
        }

        public DoubleClickAction DoubleClickAction
        {
            get { return this.doubleClickAction; }
            set { this.doubleClickAction = value; }
        }

        public int InfoPaneForeground
        {
            get { return this.infoPaneForeground.ToArgb(); }
            set { this.infoPaneForeground = Color.FromArgb(value); }
        }

        public int InfoPaneBackground
        {
            get { return this.infoPaneBackground.ToArgb(); }
            set { this.infoPaneBackground = Color.FromArgb(value); }
        }

        public int TreeViewForeground
        {
            get { return this.treeViewForeground.ToArgb(); }
            set { this.treeViewForeground = Color.FromArgb(value); }
        }

        public int TreeViewBackground
        {
            get { return this.treeViewBackground.ToArgb(); }
            set { this.treeViewBackground = Color.FromArgb(value); }
        }

        public int ReportSinkForeground
        {
            get { return this.reportSinkForeground.ToArgb(); }
            set { this.reportSinkForeground = Color.FromArgb(value); }
        }

        public int ReportSinkBackground
        {
            get { return this.reportSinkBackground.ToArgb(); }
            set { this.reportSinkBackground = Color.FromArgb(value); }
        }

        public LoggingSettings Logging
        {
            get { return this.logging; }
            set { this.logging.Copy(value); }
        }

        public ulong ReportMaxEntries
        {
            get { return this.reportMaxEntries; }
            set { this.reportMaxEntries = value; }
        }

        public static string Executable
        {
            get { return Path.GetFileName(Assembly.GetExecutingAssembly().Location); }
        }

        public static string Location
        {
            get { return Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location); }
        }

        public static string Configuration
        {
            get { return Path.Combine(ApplicationSettings.Location, Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().Location)) + ".cfg"; }
        }

        public static string RelativePath(string fullpath)
        {
            return ApplicationSettings.RelativePath(fullpath, ApplicationSettings.Location);
        }

        public static string RelativePath(string fullpath, string basepath)
        {
            if (!String.IsNullOrEmpty(basepath))
            {
                // Return an empty path if fullpath is not set.
                if (String.IsNullOrEmpty(fullpath))
                {
                    return "";
                }
                // Return relative path if fullpath is part of basepath
                else if (fullpath.StartsWith(basepath))
                {
                    string result = fullpath.Substring(basepath.Length);
                    if (result.StartsWith(Path.DirectorySeparatorChar + ""))
                    {
                        return result.Substring(1); // Remove leading backslash.
                    }
                    else
                    {
                        return result;
                    }
                }
                // Otherwise return absolute path because fullpath is not a part of basepath
                else
                {
                    return fullpath;
                }
            }
            else
            {
                throw new ArgumentException();
            }
        }

        public static string AbsolutPath(string pathpart)
        {
            return ApplicationSettings.AbsolutPath(pathpart, ApplicationSettings.Location);
        }

        public static string AbsolutPath(string pathpart, string basepath)
        {
            if (!String.IsNullOrEmpty(basepath))
            {
                // Return with basepath if pathpart is not set.
                if (String.IsNullOrEmpty(pathpart))
                {
                    return basepath;
                }
                // Combine basepath and pathpart if pathpart is a relative path
                else if (String.IsNullOrEmpty(Path.GetPathRoot(pathpart)))
                {
                    return Path.Combine(basepath, pathpart);
                }
                // Otherwise return absolute path because pathpart is not a relative path
                else
                {
                    return pathpart;
                }
            }
            else
            {
                throw new ArgumentException();
            }
        }

        public void Serialize()
        {
            this.Serialize(ApplicationSettings.Configuration);
        }

        public void Serialize(string filename)
        {
            TextWriter writer = null;
            this.serializing = true;
            try
            {
                writer = new StreamWriter(filename);
                XmlSerializer serializer = new XmlSerializer(typeof(ApplicationSettings));
                serializer.Serialize(writer, this);
            }
#if DEBUG
            catch (Exception exception)
            {
                System.Windows.Forms.MessageBox.Show(exception.ToString(), "Error (Serialize)");
#else
            catch (Exception /*exception*/)
            {
#endif
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
                this.serializing = false;
            }
        }

        public void Deserialize()
        {
            this.Deserialize(ApplicationSettings.Configuration);
        }

        public void Deserialize(string filename)
        {
            TextReader reader = null;
            try
            {
                this.loaded = false;
                reader = new StreamReader(filename);
                XmlSerializer serializer = new XmlSerializer(typeof(ApplicationSettings));
                theInstance = (ApplicationSettings)serializer.Deserialize(reader);
                this.loaded = true;
            }
#if DEBUG
            catch (FileNotFoundException /*exception*/)
            {
                // Do not show this kind of exception.
            }
            catch (Exception exception)
            {
                System.Windows.Forms.MessageBox.Show(exception.ToString(), "Error (Deserialize)");
#else
            catch (Exception /*exception*/)
            {
#endif
                this.InitializeDefaults();
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public void MRUProjectsAdd(string fullname)
        {
            if (!String.IsNullOrEmpty(fullname))
            {
                if (!this.mruProjects.Contains(fullname))
                {
                    // Remove last entries if limit has been reached!
                    if (this.mruMaxProjects > 0 && this.mruProjects.Count >= this.mruMaxProjects)
                    {
                        for (int index = this.mruMaxProjects - 1; index < this.mruProjects.Count; index++)
                        {
                            this.mruProjects.RemoveAt(index);
                        }
                    }
                }
                else
                {
                    // Remove entry at current position to put it to the front!
                    this.mruProjects.RemoveAt(this.mruProjects.IndexOf(fullname));
                }

                // Add or readd given file nmae.
                this.mruProjects.Insert(0, fullname);
            }
        }

        public void MRUProjectsDel(string fullname)
        {
            if (!String.IsNullOrEmpty(fullname))
            {
                if (this.mruProjects.Contains(fullname))
                {
                    // Remove entry at current position!
                    this.mruProjects.RemoveAt(this.mruProjects.IndexOf(fullname));
                }
            }
        }

        private void InitializeDefaults()
        {
            this.lastProjectPath = "";
            this.lastProjectFile = "";
            this.lastImportFilePath = "";
            this.lastImportFolderPath = "";
            this.mruProjects = new List<string>();
            this.mruMaxProjects = 4; // REMARK: Zero means unlimited!
            this.mainFormLocation = new Point(50, 50);
            this.mainFormDimension = new Size(680, 550);
            this.mainFormWindowState = FormWindowState.Normal;
            this.autoOpenLastProject = false;
            this.autoFBlockConnection = false;
            this.useFBlockIDNameLabel = true;
            this.reportShowErrors = true;
            this.reportShowWarnings = true;
            this.reportShowMessages = true;
            this.mainSplitDistance = 250;
            this.treeSplitDistance = 190;
            this.autoScroll = true;
            this.singleton = true;
            this.doubleClickAction = DoubleClickAction.ToggleUIVisibility;
            this.infoPaneForeground = Color.Black;
            this.infoPaneBackground = Color.White;
            this.treeViewForeground = Color.Black;
            this.treeViewBackground = Color.White;
            this.reportSinkForeground = Color.Black;
            this.reportSinkBackground = Color.White;
            this.logging = new LoggingSettings();
            this.reportMaxEntries = 1000000;
        }

        public class LoggingSettings
        {
            private string filename;
            private bool enabled;
            private bool ensure;
            private bool overwrite;
            private bool timestamp;
            private bool labels;

            public LoggingSettings()
                : base()
            {
                this.filename = "fblock.manager.log";
                this.enabled = true;
                this.ensure = false;
                this.overwrite = false;
                this.timestamp = true;
                this.labels = true;
            }

            public LoggingSettings(string filename)
                : this()
            {
                this.filename = filename;
            }

            public LoggingSettings(string filename, bool enabled)
                : this(filename)
            {
                this.enabled = enabled;
            }

            public LoggingSettings(LoggingSettings other)
                : this()
            {
                this.Copy(other);
            }

            public void Copy(LoggingSettings other)
            {
                this.filename = other.filename;
                this.enabled = other.enabled;
                this.ensure = other.ensure;
                this.overwrite = other.overwrite;
                this.timestamp = other.timestamp;
                this.labels = other.labels;
            }

            public string Filename
            {
                get { return this.filename; }
                set { this.filename = value; }
            }

            public bool Enabled
            {
                get { return this.enabled; }
                set { this.enabled = value; }
            }

            public bool Ensure
            {
                get { return this.ensure; }
                set { this.ensure = value; }
            }

            public bool Overwrite
            {
                get { return this.overwrite; }
                set { this.overwrite = value; }
            }

            public bool Timestamp
            {
                get { return this.timestamp; }
                set { this.timestamp = value; }
            }

            public bool Labels
            {
                get { return this.labels; }
                set { this.labels = value; }
            }
        }
    }
}

// EOF