﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace fblock.manager
{
    public partial class ApplicationSettingsDialog : Form
    {
        private MainForm parent = null;
        private ApplicationSettings settings = null;
        private bool canceled = true;
        private string shortcut = null;
        private string executable = null;

        public ApplicationSettingsDialog(MainForm parent)
        {
            this.parent = parent;
            this.InitializeComponent();
            this.PostInitializeComponent();
        }

        public ApplicationSettings Settings
        {
            get { return this.settings; }
        }

        private void PostInitializeComponent()
        {
            // Setup Double Click Action combo box
            List<DoubleClickActionHelper> temp = new List<DoubleClickActionHelper>();
            temp.Add(new DoubleClickActionHelper(DoubleClickAction.Nothing));
            temp.Add(new DoubleClickActionHelper(DoubleClickAction.ToggleUIVisibility));
            temp.Add(new DoubleClickActionHelper(DoubleClickAction.ToggleConnectState));
            this.cmbDoubleClickAction.DataSource = temp;
            this.cmbDoubleClickAction.DisplayMember = "Description";
            this.cmbDoubleClickAction.ValueMember = "Action";

            // Create a copy of current settings
            this.settings = ApplicationSettings.CreateCopy();

            // Recent project settings
            this.numMaxMRUProjects.Value = this.settings.MRUMaxProjects;
            this.chkClearMRUProjects.Enabled = (this.settings.MRUProjects.Length > 0);

            // General path settings
            this.chkClearLastUsedProject.Enabled = !String.IsNullOrEmpty(this.settings.LastProjectFile);
            this.chkClearLastImportedFile.Enabled = !ApplicationSettings.Location.Equals(this.settings.LastFileImport);
            this.chkClearLastImportedFolder.Enabled = !ApplicationSettings.Location.Equals(this.settings.LastFolderImport);

            // General project settings
            this.chkAutoOpenLastProject.Checked = this.settings.AutoOpenLastProject;
            this.chkAutoFBlockConnection.Checked = this.settings.AutoFBlockConnection;
            this.chkUseFBlockIDNameLabel.Checked = this.settings.UseFBlockIDNameLabel;
            this.cmbDoubleClickAction.SelectedValue = this.settings.DoubleClickAction;

            // General color settings.
            this.grpColorSettings.Enabled = (this.parent != null) ? true : false;
            if (this.parent != null)
            {
                this.btnInfoPaneForeground.Color = Color.FromArgb(this.settings.InfoPaneForeground);
                this.btnInfoPaneBackground.Color = Color.FromArgb(this.settings.InfoPaneBackground);

                this.btnTreeViewForeground.Color = Color.FromArgb(this.settings.TreeViewForeground);
                this.btnTreeViewBackground.Color = Color.FromArgb(this.settings.TreeViewBackground);

                this.btnReportSinkForeground.Color = Color.FromArgb(this.settings.ReportSinkForeground);
                this.btnReportSinkBackground.Color = Color.FromArgb(this.settings.ReportSinkBackground);
            }

            // Initialize some helper strings.
            this.shortcut = AboutDialog.AssemblyProduct;
            this.executable = Application.ExecutablePath;

            // Initialize project file association control.
            this.chkAssociateProjectFile.Checked = FileAssociation.IsAssociated();

            // Initialize Desktop shortcut control.
            this.chkPutDesktopShortcut.Checked = ShellLink.IsOnDesktop(this.shortcut);

            // Initialize SendTo shortcut control.
            this.chkAddSendToShortcut.Checked = ShellLink.IsOnSendTo(this.shortcut);

            // Initialize number of maximum report sink entries.
            this.numReportMaxEntries.Value = (this.settings.ReportMaxEntries <= this.numReportMaxEntries.Maximum) ? this.settings.ReportMaxEntries : this.numReportMaxEntries.Maximum;
        }

        private void btnSave_Click(object sender, EventArgs args)
        {
            // Recent project settings
            this.settings.MRUMaxProjects = (int)this.numMaxMRUProjects.Value;

            if (this.chkClearMRUProjects.Checked)
            {
                this.settings.MRUProjects = null;
            }

            // General path settings
            if (this.chkClearLastUsedProject.Checked)
            {
                this.settings.LastProjectPath = "";
                this.settings.LastProjectFile = "";
            }

            if (this.chkClearLastImportedFile.Checked)
            {
                this.settings.LastFileImport = "";
            }

            if (this.chkClearLastImportedFolder.Checked)
            {
                this.settings.LastFolderImport = "";
            }

            // General project settings
            this.settings.AutoOpenLastProject = this.chkAutoOpenLastProject.Checked;
            this.settings.AutoFBlockConnection = this.chkAutoFBlockConnection.Checked;
            this.settings.UseFBlockIDNameLabel = this.chkUseFBlockIDNameLabel.Checked;
            this.settings.DoubleClickAction = (DoubleClickAction)this.cmbDoubleClickAction.SelectedValue;

            // Finally save current color settings.
            if (this.parent != null)
            {
                this.parent.SetInfoPaneColor(this.btnInfoPaneForeground.Color, this.btnInfoPaneBackground.Color);
                this.settings.InfoPaneForeground = this.btnInfoPaneForeground.Color.ToArgb();
                this.settings.InfoPaneBackground = this.btnInfoPaneBackground.Color.ToArgb();

                this.parent.SetTreeViewColor(this.btnTreeViewForeground.Color, this.btnTreeViewBackground.Color);
                this.settings.TreeViewForeground = this.btnTreeViewForeground.Color.ToArgb();
                this.settings.TreeViewBackground = this.btnTreeViewBackground.Color.ToArgb();

                this.parent.SetReportSinkColor(this.btnReportSinkForeground.Color, this.btnReportSinkBackground.Color);
                this.settings.ReportSinkForeground = this.btnReportSinkForeground.Color.ToArgb();
                this.settings.ReportSinkBackground = this.btnReportSinkBackground.Color.ToArgb();
            }

            // Create or remove project file association.
            if (this.chkAssociateProjectFile.Checked && !FileAssociation.IsAssociated())
            {
                FileAssociation.Associate(this.parent.ReportSink);
            }
            else if (!this.chkAssociateProjectFile.Checked && FileAssociation.IsAssociated())
            {
                FileAssociation.Remove(this.parent.ReportSink);
            }

            // Create or remove Desktop shortcut.
            if (this.chkPutDesktopShortcut.Checked && !ShellLink.IsOnDesktop(this.shortcut))
            {
                ShellLink.AddDesktop(this.shortcut, this.executable);
            }
            else if (!this.chkPutDesktopShortcut.Checked && ShellLink.IsOnDesktop(this.shortcut))
            {
                ShellLink.DelDesktop(this.shortcut, this.executable);
            }

            // Create or remove SendTo shortcut.
            if (this.chkAddSendToShortcut.Checked && !ShellLink.IsOnSendTo(this.shortcut))
            {
                ShellLink.AddSendTo(this.shortcut, this.executable);
            }
            else if (!this.chkAddSendToShortcut.Checked && ShellLink.IsOnSendTo(this.shortcut))
            {
                ShellLink.DelSendTo(this.shortcut, this.executable);
            }

            // Save number of maximum report sink entries.
            this.settings.ReportMaxEntries = (uint)this.numReportMaxEntries.Value;

            this.canceled = false;
        }

        private void btnCancel_Click(object sender, EventArgs args)
        {
            this.canceled = true;
        }

        private void btnLogging_Click(object sender, EventArgs args)
        {
            LoggingDialog dialog = new LoggingDialog(this.settings.Logging, true);
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                // Take over made changes!
                this.settings.Logging.Copy(dialog.Settings);
            }
        }

        private void ApplicationSettingsDialog_FormClosed(object sender, FormClosedEventArgs args)
        {
            // Restore original colors if dialog was closed without saving changes.
            if (this.canceled && this.parent != null)
            {
                this.parent.SetInfoPaneColor(
                    Color.FromArgb(this.settings.InfoPaneForeground),
                    Color.FromArgb(this.settings.InfoPaneBackground)
                );

                this.parent.SetTreeViewColor(
                    Color.FromArgb(this.settings.TreeViewForeground),
                    Color.FromArgb(this.settings.TreeViewBackground)
                );

                this.parent.SetReportSinkColor(
                    Color.FromArgb(this.settings.ReportSinkForeground),
                    Color.FromArgb(this.settings.ReportSinkBackground)
                );
            }
        }

        private void chkClearLastUsedProject_MouseHover(object sender, EventArgs args)
        {
            this.tipGeneral.SetToolTip(this.chkClearLastUsedProject, this.settings.LastProject);
        }

        private void chkClearLastImportedFile_MouseHover(object sender, EventArgs args)
        {
            this.tipGeneral.SetToolTip(this.chkClearLastImportedFile, this.settings.LastFileImport);
        }

        private void chkClearLastImportedFolder_MouseHover(object sender, EventArgs args)
        {
            this.tipGeneral.SetToolTip(this.chkClearLastImportedFolder, this.settings.LastFolderImport);
        }

        private void btnInfoPaneForeground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnInfoPaneForeground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnInfoPaneForeground.Color = dialog.Color;
            }
        }

        private void btnInfoPaneBackground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnInfoPaneBackground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnInfoPaneBackground.Color = dialog.Color;
            }
        }

        private void btnInfoPaneColorPreview_Click(object sender, EventArgs args)
        {
            this.parent.SetInfoPaneColor(this.btnInfoPaneForeground.Color, this.btnInfoPaneBackground.Color);
        }

        private void btnTreeViewForeground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnTreeViewForeground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnTreeViewForeground.Color = dialog.Color;
            }
        }

        private void btnTreeViewBackground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnTreeViewBackground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnTreeViewBackground.Color = dialog.Color;
            }
        }

        private void btnTreeViewColorPreview_Click(object sender, EventArgs args)
        {
            this.parent.SetTreeViewColor(this.btnTreeViewForeground.Color, this.btnTreeViewBackground.Color);
        }

        private void btnReportSinkForeground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnReportSinkForeground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnReportSinkForeground.Color = dialog.Color;
            }
        }

        private void btnReportSinkBackground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnReportSinkBackground.Color;
            if (DialogResult.OK == dialog.ShowDialog())
            {
                this.btnReportSinkBackground.Color = dialog.Color;
            }
        }

        private void btnReportSinkColorPreview_Click(object sender, EventArgs args)
        {
            this.parent.SetReportSinkColor(this.btnReportSinkForeground.Color, this.btnReportSinkBackground.Color);
        }
    }
}

// EOF