/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;

namespace fblock.manager
{
    public class CommandLineParser
    {
        private const string OPT_INITIALIZE = "initialize";
        private const string OPT_AUTOLOAD = "autoload";
        private const string OPT_CONNECT = "connect";
        private const string OPT_HIDE = "hide";
        private const string OPT_FOLDER = "folder";
        private const string OPT_PROJECT = "project";
        private const string OPT_LOGFILE = "logfile";
        private const string OPT_OVERWRITE = "overwrite";
        private const string OPT_SHUTDOWN = "shutdown";
        private const string OPT_NOSAVE = "nosave";

        private bool initialize = false;
        private bool autoload = false;
        private bool connect = false;
        private bool hide = false;
        private string folder = ""; // Same folder as the FBlockManager!
        private string project = "rbs_default.fblock";
        private string input = "";
        private string logfile = "";
        private bool overwrite = false;
        private bool shutdown = false;
        private bool nosave = false;

        public CommandLineParser()
            : base()
        {
        }

        public CommandLineParser(string[] arguments)
            : this()
        {
            this.Parse(arguments);
        }

        #region Command Line Parser Properties

        public bool Initialize
        {
            get { return this.initialize; }
        }

        public bool Autoload
        {
            get { return this.autoload; }
        }

        public bool Connect
        {
            get { return this.connect; }
        }

        public bool Hide
        {
            get { return this.hide; }
        }

        public string Folder
        {
            get { return this.folder; }
        }

        public string Project
        {
            get { return this.project; }
        }

        public bool IsLogging
        {
            get { return !String.IsNullOrEmpty(this.logfile); }
        }

        public string Logfile
        {
            get { return this.logfile; }
        }

        public bool Overwrite
        {
            get { return this.overwrite; }
        }

        public bool Shutdown
        {
            get { return this.shutdown; }
        }

        public bool NoSave
        {
            get { return this.nosave; }
        }

        public string InputString
        {
            get { return this.input; }
        }

        public string InputFile
        {
            get
            {
                try
                {
                    return Path.GetFullPath(this.input);
                }
                catch (Exception exception)
                {
                    System.Diagnostics.Debug.WriteLine(exception.ToString());
                    return "";
                }
            }
        }

        #endregion // Command Line Parser Properties

        private void Parse(string[] args)
        {
            // Put everything in one single string!
            string arguments = "";
            foreach (string temp in args)
            {
                arguments += temp + " ";
            }

            // Just save current argument list and assume it as a single file name!
            this.input = arguments.Trim();

            // Analyze every character within given argument list
            char[] array = arguments.ToCharArray();
            for (int index = 0; index < array.Length; index++)
            {
                string option = "";
                string value = "";

                if (array[index] == '/') // Option start
                {
                    // Read current option name
                    while (index < array.Length && array[index] != ' ')
                    {
                        option += array[++index];
                    }
                    option = option.Trim();

                    // Read current option value
                    while (index < array.Length && array[index] != '/')
                    {
                        value += array[index++];
                    }
                    value = value.Trim();

                    index--;

                    if (option.ToLower() == CommandLineParser.OPT_INITIALIZE)
                    {
                        this.initialize = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_AUTOLOAD)
                    {
                        this.autoload = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_CONNECT)
                    {
                        this.connect = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_HIDE)
                    {
                        this.hide = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_FOLDER)
                    {
                        this.folder = value;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                        this.input = this.Remove(this.input, value);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_PROJECT)
                    {
                        this.project = value;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                        this.input = this.Remove(this.input, value);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_LOGFILE)
                    {
                        this.logfile = Path.GetFullPath(value);

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                        this.input = this.Remove(this.input, value);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_OVERWRITE)
                    {
                        this.overwrite = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_SHUTDOWN)
                    {
                        this.shutdown = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                    else if (option.ToLower() == CommandLineParser.OPT_NOSAVE)
                    {
                        this.nosave = true;

                        // Adjust input to use the rest as project file.
                        this.input = this.Remove(this.input, "/" + option);
                    }
                }
            }
        }

        private string Remove(string source, string remove)
        {
            return source.Remove(source.IndexOf(remove), remove.Length).Trim();
        }

        public override string ToString()
        {
            return base.ToString() + ":" +
                " Initialize=" + this.Initialize.ToString().ToUpper() +
                " Autoload=" + this.Autoload.ToString().ToUpper() +
                " Folder=\"" + this.Folder + "\"" +
                " Project=\"" + this.Project + "\"";
        }
    }
}

// EOF