﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;

namespace fblock.manager
{
    public class FBlockTreeView : TreeView
    {
        public event EventHandler<EventArgs> CurrentProjectChanged;

        private IReportSink reportSinkPanel = null;
        private TreeNode rootNode = null;
        private TreeNode dragNode = null;
        private string currentProject = null;
        private ProjectSettings projectSettings = null;
        private bool dragging = false;

        private enum IconKeys
        {
            FBlockRoot = 1,
            FBlockShowUI = 2,
            FBlockShowUIPlay = 3,
            FBlockHideUI = 4,
            FBlockHideUIPlay = 5,
        }

        public FBlockTreeView()
            : base()
        {
            this.InitializeComponent();
            this.PostInitializeComponent();
        }

        public IReportSink ReportSink
        {
            get { return this.reportSinkPanel; }
            set { this.reportSinkPanel = value; }
        }

        public bool SaveAsEnabled
        {
            get { return (!String.IsNullOrEmpty(this.currentProject)); }
        }

        public bool CloseEnabled
        {
            get { return (!String.IsNullOrEmpty(this.currentProject)); }
        }

        public bool ProjectSettingsEnabled
        {
            get { return (this.projectSettings != null); }
        }

        public ProjectSettings CurrentProjectSettings
        {
            get { return this.projectSettings; }
        }

        public string CurrentProject
        {
            get
            {
                if (String.IsNullOrEmpty(this.currentProject))
                {
                    return "";
                }
                else
                {
                    FileInfo fileInfo = new FileInfo(this.currentProject);
                    return fileInfo.Name;
                }
            }
            set
            {
                this.currentProject = value;

                EventHandler<EventArgs> handler = CurrentProjectChanged;
                if (handler != null)
                {
                    handler(this, new EventArgs());
                }
            }
        }

        public bool IsDragging
        {
            get { return this.dragging; }
        }

        public void UpdateParent()
        {
            // This code forces an AfterSelect() event and 
            // therefore the main window will be updated!
            TreeNode current = this.SelectedNode;
            this.SelectedNode = null;
            this.SelectedNode = current;
        }

        public void AutoConnect()
        {
            if (this.rootNode != null && ApplicationSettings.Instance.AutoFBlockConnection)
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        if (!fblock.IsConnected)
                        {
                            fblock.Connect();
                        }
                    }
                }
            }
        }

        public void ApplicationSettingsChanged()
        {
            if (this.rootNode != null)
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        // Re-new current node name depending on current settings.
                        node.Text = this.NodeName((FBlockInvoker)node.Tag);
                    }
                }
            }
        }

        public bool ImportFromFile()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Multiselect = true;
            dialog.Filter = "FBlock binaries (*.dll,*.exe)|*.dll;*.exe|FBlock libraries (*.dll)|*.dll|FBlock executables (*.exe)|*.exe|All files (*.*)|*.*";
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.RestoreDirectory = false;
            dialog.InitialDirectory = ApplicationSettings.Instance.LastFileImport;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                Program.Instance.MainForm.Redraw();

                Logging.Instance.Message("----- START IMPORT FROM FILE -----");

                if (this.AddItems(dialog.FileNames)) // True if at least one filename is available.
                {
                    ApplicationSettings.Instance.LastFileImport = Path.GetDirectoryName(dialog.FileName);
                    this.projectSettings = new ProjectSettings();

                    Logging.Instance.Message("----- END IMPORT FROM FILE -----");

                    return true;
                }
            }
            Program.Instance.MainForm.Redraw();
            return false;
        }

        public bool ImportFromFolder()
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            dialog.Description = "\nSelect a folder to import FBlock binaries from.";
            dialog.ShowNewFolderButton = false;
            dialog.SelectedPath = ApplicationSettings.Instance.LastFolderImport;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                Program.Instance.MainForm.Redraw();

                Logging.Instance.Message("----- START IMPORT FROM FOLDER -----");

                ApplicationSettings.Instance.LastFolderImport = dialog.SelectedPath; // Always save last path.

                this.AddItems(Directory.GetFiles(ApplicationSettings.Instance.LastFolderImport, "*.dll"));
                this.AddItems(Directory.GetFiles(ApplicationSettings.Instance.LastFolderImport, "*.exe"));
                this.projectSettings = new ProjectSettings();

                Logging.Instance.Message("----- END IMPORT FROM FOLDER -----");

                return true;
            }
            Program.Instance.MainForm.Redraw();
            return false;
        }

        public bool OpenProject()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Multiselect = true;
            dialog.Filter = "FBlock projects (*.fblock)|*.fblock|All files (*.*)|*.*";
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.RestoreDirectory = false;
            dialog.InitialDirectory = ApplicationSettings.Instance.LastProjectPath;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                Program.Instance.MainForm.Redraw();
                this.CloseProject();
                return this.OpenProject(dialog.FileName);
            }
            Program.Instance.MainForm.Redraw();
            return false;
        }

        public bool OpenProject(string fullname)
        {
            Cursor cursor = Program.Instance.MainForm.Cursor;
            Program.Instance.MainForm.Cursor = Cursors.WaitCursor;

            try
            {
                if (this.rootNode == null)
                {
                    if (File.Exists(fullname))
                    {
                        Logging.Instance.Message("----- START OPEN PROJECT [" + Path.GetFileName(fullname) + "] -----");

                        ProjectSerializer serializer = new ProjectSerializer(this.reportSinkPanel);

                        if (serializer.Deserialize(fullname))
                        {
                            foreach (ProjectEntry entry in serializer.Settings.ProjectEntrys)
                            {
                                TreeNode node = this.AddItem(entry.Assembly);

                                if (node != null && node.Tag != null)
                                {
                                    FBlockInvoker fblock = (FBlockInvoker)node.Tag;

                                    // Always use settings depending node name.
                                    node.Text = this.NodeName(fblock);

                                    this.SetSelected(node, entry.Selected);

                                    fblock.MainForm.Location = this.BoundsToScreen(entry.Location, fblock.MainForm.Size);

                                    if (Program.Instance.MainForm.CommandLineHide)
                                    {
                                        fblock.ShowUI = false;
                                    }
                                    else
                                    {
                                        fblock.ShowUI = entry.ShowUI;
                                    }

                                    if (entry.Connected)
                                    {
                                        fblock.Connect();
                                    }
                                    else
                                    {
                                        fblock.Disconnect();
                                    }
                                }
                            }

                            // But only if successful!
                            ApplicationSettings.Instance.LastProject = fullname;
                            ApplicationSettings.Instance.MRUProjectsAdd(fullname);

                            this.CurrentProject = fullname; // Save current project file name and path.
                            this.projectSettings = serializer.Settings;

                            Logging.Instance.Message("----- END OPEN PROJECT [" + Path.GetFileName(fullname) + "] -----");

                            return true;
                        }
                    }
                    else
                    {
                        if (this.reportSinkPanel != null)
                        {
                            this.reportSinkPanel.Warning("Project file not found!", "FBlockManager", "Open", (!String.IsNullOrEmpty(fullname) ? fullname : "<empty>"));
                        }
                        ApplicationSettings.Instance.MRUProjectsDel(fullname);
                    }
                }
                return false;
            }
            catch (Exception /*exception*/)
            {
                return false;
            }
            finally
            {
                Program.Instance.MainForm.Cursor = cursor;
            }
        }

        public bool SaveProject()
        {
            bool success = false;

            if (String.IsNullOrEmpty(this.currentProject))
            {
                string temp = ApplicationSettings.Instance.LastProjectFile;
                ApplicationSettings.Instance.LastProjectFile = "";

                success = this.SaveProjectAs();

                if (!success)
                {
                    ApplicationSettings.Instance.LastProjectFile = temp;
                }
            }
            else
            {
                success = this.SaveProject(this.currentProject);
            }

            return success;
        }

        public bool SaveProjectAs()
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "FBlock projects (*.fblock)|*.fblock|All files (*.*)|*.*";
            dialog.CheckFileExists = false;
            dialog.CheckPathExists = true;
            dialog.RestoreDirectory = false;
            dialog.FileName = ApplicationSettings.Instance.LastProjectFile;
            dialog.InitialDirectory = ApplicationSettings.Instance.LastProjectPath;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                Program.Instance.MainForm.Redraw();
                return this.SaveProject(dialog.FileName);
            }
            Program.Instance.MainForm.Redraw();
            return false;
        }

        private bool SaveProject(string fullname)
        {
            Cursor cursor = Program.Instance.MainForm.Cursor;
            Program.Instance.MainForm.Cursor = Cursors.WaitCursor;

            try
            {
                if (this.rootNode != null)
                {
                    this.projectSettings.ProjectEntrys.Clear();
                    foreach (TreeNode node in this.rootNode.Nodes)
                    {
                        if (node.Tag != null)
                        {
                            ProjectEntry entry = new ProjectEntry();
                            FBlockInvoker fblock = (FBlockInvoker)node.Tag;

                            // Always save FBlock ID with FBlock name.
                            entry.Name = this.NodeName(fblock, true);
                            entry.Assembly = fblock.AssemblyName;
                            entry.Location = fblock.MainForm.Location;
                            entry.ShowUI = fblock.ShowUI;
                            entry.Connected = fblock.IsConnected;
                            entry.Selected = node.Checked;

                            this.projectSettings.ProjectEntrys.Add(entry);
                        }
                    }

                    // Save last saved date/time.
                    this.projectSettings.Saved = DateTime.Now;

                    // Save current project to given file.
                    ProjectSerializer serializer = new ProjectSerializer(this.reportSinkPanel);
                    serializer.Settings = this.projectSettings;
                    if (serializer.Serialize(fullname))
                    {
                        ApplicationSettings.Instance.LastProject = fullname;
                        ApplicationSettings.Instance.MRUProjectsAdd(fullname);

                        this.CurrentProject = fullname; // Save current project file name and path.

                        return true;
                    }
                }
                return false;
            }
            catch (Exception /*exception*/)
            {
                return false;
            }
            finally
            {
                Program.Instance.MainForm.Cursor = cursor;
            }
        }

        public void CloseProject()
        {
            if (this.rootNode != null)
            {
                Logging.Instance.Message("----- START CLOSE PROJECT -----");

                TreeNode node = this.rootNode.FirstNode;
                while (node != null)
                {
                    TreeNode next = node.NextNode;
                    this.DelItem(node);
                    node = next;
                }

                Logging.Instance.Message("----- END CLOSE PROJECT -----");
            }

            this.CurrentProject = null; // Reset current project file name and path.
            this.projectSettings = null; // Reset current project settings too.
        }

        private bool AddItems(string[] filenames)
        {
            int failed = 0; // Count number of files which fail to add!

            foreach (string filename in filenames)
            {
                if (this.AddItem(filename) == null)
                {
                    failed++;
                }
            }

            return (filenames.Length != 0 && failed == 0);
        }

        private TreeNode AddItem(string filename)
        {
            Cursor cursor = Program.Instance.MainForm.Cursor;
            Program.Instance.MainForm.Cursor = Cursors.WaitCursor;

            try
            {
                // Load assembly
                FBlockInvoker fblock = new FBlockInvoker(filename, this.reportSinkPanel);
                if (fblock.IsValid)
                {
                    if (fblock.Initialize(this.reportSinkPanel))
                    {
                        // Connect to event handlers
                        IFBlockAccessorEvents events = fblock.GetEvents();
                        if (events != null)
                        {
                            events.Connected += new EventHandler<EventArgs>(events_Connected);
                            events.Disconnected += new EventHandler<EventArgs>(events_Disconnected);
                            events.InstanceIDChanged += new EventHandler<InstanceIDChangedEventArgs>(events_InstanceIDChanged);
                            events.ShowUIChanged += new EventHandler<ShowUIChangedEventArgs>(events_ShowUIChanged);
                            events.LocationChanged += new EventHandler<EventArgs>(events_LocationChanged);
                        }

                        return this.AddItem(fblock);
                    }
                }
                else
                {
                    throw new InvalidOperationException("Requested interface not supported.");
                }
            }
            catch (Exception exception)
            {
                if (this.reportSinkPanel != null)
                {
                    FileInfo fileInfo = new FileInfo(filename);
                    this.reportSinkPanel.Warning("File \"" + fileInfo.Name + "\" could not be loaded!", "FBlockManager", "Open", exception.Message);
                }
            }
            finally
            {
                Program.Instance.MainForm.Cursor = cursor;
            }
            return null;
        }

        private string NodeName(FBlockInvoker fblock, bool useFBlockID)
        {
            if (useFBlockID)
            {
                return string.Format("{0} (0x{1:X4})", fblock.Name, fblock.FBlockID);
            }
            else
            {
                return string.Format("{0}", fblock.Name);
            }
        }

        private string NodeName(FBlockInvoker fblock)
        {
            return this.NodeName(fblock, ApplicationSettings.Instance.UseFBlockIDNameLabel);
        }

        private TreeNode AddItem(FBlockInvoker fblock)
        {
            if (this.Nodes.Count == 0)
            {
                this.rootNode = new TreeNode("FBlocks");
                this.Nodes.Add(this.rootNode);
                this.rootNode.StateImageKey = IconKeys.FBlockRoot.ToString();
            }
            else
            {
                this.rootNode = this.Nodes[0];
            }

            TreeNode node = new TreeNode(this.NodeName(fblock));
            this.rootNode.Nodes.Add(node);
            node.ImageKey = this.GetImageKey(fblock);
            node.SelectedImageKey = node.ImageKey;

            node.Tag = fblock;
            node.EnsureVisible();
            node.NodeFont = new Font(this.Font, this.Font.Style);
            this.SelectedNode = node;

            return node;
        }

        private void DelItem(TreeNode node)
        {
            try
            {
                if (node != null)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.Disconnect();
                        fblock.Cleanup();

                        // Disconnect from event handlers
                        IFBlockAccessorEvents events = fblock.GetEvents();
                        if (events != null)
                        {
                            events.Connected -= new EventHandler<EventArgs>(events_Connected);
                            events.Disconnected -= new EventHandler<EventArgs>(events_Disconnected);
                            events.InstanceIDChanged -= new EventHandler<InstanceIDChangedEventArgs>(events_InstanceIDChanged);
                            events.ShowUIChanged -= new EventHandler<ShowUIChangedEventArgs>(events_ShowUIChanged);
                            events.LocationChanged -= new EventHandler<EventArgs>(events_LocationChanged);
                        }
                    }

                    // Remove given node from tree view.
                    node.Remove();

                    // Remove root node if not other children are available.
                    if (this.rootNode.Nodes.Count == 0)
                    {
                        this.rootNode.Remove();
                        this.rootNode = null;

                        Program.Instance.MainForm.Dirty = false; // Empty project, nothing to save
                    }
                    else
                    {
                        Program.Instance.MainForm.Dirty = true;
                    }
                }
            }
            catch (Exception /*exception*/)
            {
            }
        }

        private string GetImageKey(FBlockInvoker fblock)
        {
            if (fblock.ShowUI)
            {
                if (fblock.IsConnected)
                {
                    return IconKeys.FBlockShowUIPlay.ToString();
                }
                else
                {
                    return IconKeys.FBlockShowUI.ToString();
                }
            }
            else
            {
                if (fblock.IsConnected)
                {
                    return IconKeys.FBlockHideUIPlay.ToString();
                }
                else
                {
                    return IconKeys.FBlockHideUI.ToString();
                }
            }
        }

        private TreeNode NodeFromSender(object sender)
        {
            try
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                    if (fblock.IsValid && sender.Equals(fblock.GetAccessor()))
                    {
                        return node;
                    }
                }
            }
            catch (Exception /*exception*/)
            {
            }
            return null;
        }

        private void SetSelected(TreeNode node, bool selected)
        {
            // TODO: Find a better way to adjust the label width instead of setting the text again.
            if (selected)
            {
                node.Checked = true;
                node.NodeFont = new Font(node.NodeFont, node.NodeFont.Style | FontStyle.Bold);
                node.Text = node.Text;
            }
            else
            {
                node.Checked = false;
                node.NodeFont = new Font(node.NodeFont, node.NodeFont.Style & ~FontStyle.Bold);
                node.Text = node.Text;
            }

            Program.Instance.MainForm.Dirty = true;
        }

        private Point BoundsToScreen(Point location, Size size)
        {
            Rectangle bounds = new Rectangle(location, size);

            foreach (Screen screen in Screen.AllScreens)
            {
                if (screen.Bounds.Contains(bounds))
                {
                    return bounds.Location; // Location is still within bounds.
                }
            }
            return new Point(50, 50); // Location is out of bounds, return with default location.
        }

        #region IFBlockAccessor event handlers

        private void events_Connected(object sender, EventArgs args)
        {
            TreeNode node = this.NodeFromSender(sender);
            if (node != null)
            {
                node.ImageKey = this.GetImageKey((FBlockInvoker)node.Tag);
                node.SelectedImageKey = node.ImageKey;

                if (this.reportSinkPanel != null)
                {
                    this.reportSinkPanel.Message("Module [" + node.Text + "] has been connected.", "FBlockManager", "Event");
                }

                this.UpdateParent();

                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void events_Disconnected(object sender, EventArgs args)
        {
            TreeNode node = this.NodeFromSender(sender);
            if (node != null)
            {
                node.ImageKey = this.GetImageKey((FBlockInvoker)node.Tag);
                node.SelectedImageKey = node.ImageKey;

                if (this.reportSinkPanel != null)
                {
                    this.reportSinkPanel.Message("Module [" + node.Text + "] has been disconnected.", "FBlockManager", "Event");
                }

                this.UpdateParent();

                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void events_InstanceIDChanged(object sender, InstanceIDChangedEventArgs args)
        {
            TreeNode node = this.NodeFromSender(sender);
            if (node != null)
            {
                if (this.reportSinkPanel != null)
                {
                    this.reportSinkPanel.Message("Module [" + node.Text + "] uses instance ID " + args.InstanceID + ".", "FBlockManager", "Event");
                }

                this.UpdateParent();
            }
        }

        private void events_ShowUIChanged(object sender, ShowUIChangedEventArgs args)
        {
            TreeNode node = this.NodeFromSender(sender);
            if (node != null)
            {
                node.ImageKey = this.GetImageKey((FBlockInvoker)node.Tag);
                node.SelectedImageKey = node.ImageKey;

                if (this.reportSinkPanel != null)
                {
                    if (args.StateUI == ShowUIChangedEventArgs.ShowState.Shown)
                    {
                        this.reportSinkPanel.Message("Form of module [" + node.Text + "] is shown.", "FBlockManager", "Event");
                    }
                    else if (args.StateUI == ShowUIChangedEventArgs.ShowState.Hidden)
                    {
                        this.reportSinkPanel.Message("Form of module [" + node.Text + "] is hidden.", "FBlockManager", "Event");
                    }
                    else if (args.StateUI == ShowUIChangedEventArgs.ShowState.Minimized)
                    {
                        this.reportSinkPanel.Message("Form of module [" + node.Text + "] is minimized.", "FBlockManager", "Event");
                    }
                    else if (args.StateUI == ShowUIChangedEventArgs.ShowState.Maximized)
                    {
                        this.reportSinkPanel.Message("Form of module [" + node.Text + "] is maximized.", "FBlockManager", "Event");
                    }
                }

                this.UpdateParent();

                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void events_LocationChanged(object sender, EventArgs args)
        {
            TreeNode node = this.NodeFromSender(sender);
            if (node != null)
            {
                Program.Instance.MainForm.Dirty = true;
            }
        }

        #endregion // IFBlockAccessor event handlers

        #region FBlockTreeView event handlers

        private void FBlockTreeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs args)
        {
            if (args.Button == MouseButtons.Right)
            {
                // Change selected node when right mouse button was used.
                this.SelectedNode = args.Node;
            }
            else if (args.Button == MouseButtons.Left)
            {
                if (Control.ModifierKeys == Keys.Control)
                {
                    TreeNode node = args.Node;
                    if (!node.Equals(this.rootNode))
                    {
                        this.SetSelected(node, !node.Checked);
                    }
                }
            }
        }

        private void FBlockTreeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs args)
        {
            if (!args.Node.Equals(this.rootNode))
            {
                if (args.Node.Tag != null)
                {
                    FBlockInvoker fblock = (FBlockInvoker)args.Node.Tag;
                    if (ApplicationSettings.Instance.DoubleClickAction == DoubleClickAction.ToggleConnectState)
                    {
                        if (fblock.IsConnected)
                        {
                            fblock.Disconnect();
                        }
                        else
                        {
                            fblock.Connect();
                        }
                        Program.Instance.MainForm.Dirty = true;
                    }
                    else if (ApplicationSettings.Instance.DoubleClickAction == DoubleClickAction.ToggleUIVisibility)
                    {
                        fblock.ShowUI = !fblock.ShowUI;
                        Program.Instance.MainForm.Dirty = true;
                    }
                }
            }
        }

        private void FBlockTreeView_ItemDrag(object sender, ItemDragEventArgs args)
        {
            this.dragNode = (TreeNode)args.Item;
            this.SelectedNode = this.dragNode;

            if (Control.ModifierKeys == Keys.Control)
            {
                this.DoDragDrop(args.Item.ToString(), DragDropEffects.Move);
            }
        }

        private void FBlockTreeView_DragEnter(object sender, DragEventArgs args)
        {
            if (args.Data.GetDataPresent(DataFormats.Text))
            {
                args.Effect = DragDropEffects.Move;
                this.dragging = true;
            }
            else
            {
                args.Effect = DragDropEffects.None;
                this.dragging = false;
            }
        }

        private void FBlockTreeView_DragOver(object sender, DragEventArgs args)
        {
            TreeNode current = this.GetNodeAt(this.PointToClient(new Point(args.X, args.Y)));

            // Select node item currently under the mouse.
            if (current != null)
            {
                this.SelectedNode = current;
            }
        }

        private void FBlockTreeView_DragDrop(object sender, DragEventArgs args)
        {
            TreeNode targetNode = this.GetNodeAt(this.PointToClient(new Point(args.X, args.Y)));

            if (this.rootNode != null && targetNode != null)
            {
                int sourceIndex = this.dragNode.Index;
                int targetIndex = targetNode.Index;

                // Remove node from parent node.
                this.rootNode.Nodes.RemoveAt(sourceIndex);
                // Insert node at new location.
                this.rootNode.Nodes.Insert(targetIndex, this.dragNode);

                this.SelectedNode = this.dragNode;
                this.SetSelected(this.dragNode, this.dragNode.Checked);
                this.Invalidate();
            }
            this.dragging = false;
        }

        private void ContextMenu_Opening(object sender, CancelEventArgs args)
        {
            // Enable/Disable menu items depending on 
            // current tree item count and other states.

            bool general = (this.Nodes.Count != 0);
            bool notroot = (this.SelectedNode != this.rootNode);
            bool current = (this.SelectedNode != null);
            bool selected = false;

            // Try to find at least one selected item.
            for (int index = 0; this.rootNode != null && index < this.rootNode.Nodes.Count && selected == false; index++)
            {
                selected = this.rootNode.Nodes[index].Checked;
            }

            this.menuConnect.Enabled = general;
            this.menuDisconnect.Enabled = general;
            this.menuShowUI.Enabled = general;
            this.menuHideUI.Enabled = general;
            this.menuSelectAll.Enabled = general;
            this.menuDeselectAll.Enabled = general;
            this.menuRemove.Enabled = general;

            this.menuRemoveCurrent.Enabled = (general && notroot);
            this.menuSelect.Enabled = (general && notroot);
            this.menuDeselect.Enabled = (general && notroot);

            this.menuConnectCurrent.Enabled = (current && notroot);
            this.menuDisconnectCurrent.Enabled = (current && notroot);
            this.menuShowUICurrent.Enabled = (current && notroot);
            this.menuHideUICurrent.Enabled = (current && notroot);

            this.menuConnectSelected.Enabled = selected;
            this.menuDisconnectSelected.Enabled = selected;
            this.menuShowUISelected.Enabled = selected;
            this.menuHideUISelected.Enabled = selected;
            this.menuRemoveSelected.Enabled = selected;
        }

        private void MenuConnect_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            if (sender.Equals(this.menuConnectCurrent))
            {
                if (this.SelectedNode != null && this.SelectedNode.Tag != null)
                {
                    FBlockInvoker fblock = (FBlockInvoker)this.SelectedNode.Tag;
                    fblock.Connect();
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuConnectSelected))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Checked && node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.Connect();
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuConnectAll))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.Connect();
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void MenuDisconnect_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            if (sender.Equals(this.menuDisconnectCurrent))
            {
                if (this.SelectedNode != null && this.SelectedNode.Tag != null)
                {
                    FBlockInvoker fblock = (FBlockInvoker)this.SelectedNode.Tag;
                    fblock.Disconnect();
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuDisconnectSelected))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Checked && node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.Disconnect();
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuDisconnectAll))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.Disconnect();
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void MenuShowUI_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            if (sender.Equals(this.menuShowUICurrent))
            {
                if (this.SelectedNode != null && this.SelectedNode.Tag != null)
                {
                    FBlockInvoker fblock = (FBlockInvoker)this.SelectedNode.Tag;
                    fblock.ShowUI = true;
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuShowUISelected))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Checked && node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.ShowUI = true;
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuShowUIAll))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.ShowUI = true;
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void MenuHideUI_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            if (sender.Equals(this.menuHideUICurrent))
            {
                if (this.SelectedNode != null && this.SelectedNode.Tag != null)
                {
                    FBlockInvoker fblock = (FBlockInvoker)this.SelectedNode.Tag;
                    fblock.ShowUI = false;
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuHideUISelected))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Checked && node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.ShowUI = false;
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
            else if (sender.Equals(this.menuHideUIAll))
            {
                foreach (TreeNode node in this.rootNode.Nodes)
                {
                    if (node.Tag != null)
                    {
                        FBlockInvoker fblock = (FBlockInvoker)node.Tag;
                        fblock.ShowUI = false;
                    }
                }
                Program.Instance.MainForm.Dirty = true;
            }
        }

        private void MenuSelect_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            TreeNode node = this.SelectedNode;
            if (!node.Equals(this.rootNode))
            {
                this.SetSelected(node, true);
            }
        }

        private void MenuDeselect_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            TreeNode node = this.SelectedNode;
            if (!node.Equals(this.rootNode))
            {
                this.SetSelected(node, false);
            }
        }

        private void MenuSelectAll_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            foreach (TreeNode node in this.rootNode.Nodes)
            {
                this.SetSelected(node, true);
            }
        }

        private void MenuDeselectAll_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            foreach (TreeNode node in this.rootNode.Nodes)
            {
                this.SetSelected(node, false);
            }
        }

        private void MenuRemove_Click(object sender, EventArgs args)
        {
            Program.Instance.MainForm.Redraw();

            if (sender.Equals(this.menuRemoveCurrent))
            {
                TreeNode next = this.SelectedNode.NextNode;
                if (next == null)
                {
                    next = this.rootNode;
                }
                this.DelItem(this.SelectedNode);
            }
            else if (sender.Equals(this.menuRemoveSelected))
            {
                TreeNode node = this.rootNode.FirstNode;
                while (node != null)
                {
                    if (node.Checked)
                    {
                        TreeNode next = node.NextNode;
                        this.DelItem(node);
                        node = next;
                    }
                    else
                    {
                        TreeNode next = node.NextNode;
                        node = next;
                    }
                }
            }
            else if (sender.Equals(this.menuRemoveAll))
            {
                TreeNode node = this.rootNode.FirstNode;
                while (node != null)
                {
                    TreeNode next = node.NextNode;
                    this.DelItem(node);
                    node = next;
                }
            }
        }

        #endregion // FBlockTreeView event handlers

        #region FBlockTreeView component setup

        private IContainer components;
        private ContextMenuStrip contextMenu;
        private ToolStripMenuItem menuConnect;
        private ToolStripMenuItem menuConnectCurrent;
        private ToolStripMenuItem menuConnectSelected;
        private ToolStripMenuItem menuConnectAll;
        private ToolStripMenuItem menuDisconnect;
        private ToolStripMenuItem menuDisconnectCurrent;
        private ToolStripMenuItem menuDisconnectSelected;
        private ToolStripMenuItem menuDisconnectAll;
        private ToolStripMenuItem menuShowUI;
        private ToolStripMenuItem menuShowUICurrent;
        private ToolStripMenuItem menuShowUISelected;
        private ToolStripMenuItem menuShowUIAll;
        private ToolStripSeparator menuSeparator1;
        private ToolStripMenuItem menuRemove;
        private ToolStripMenuItem menuRemoveCurrent;
        private ToolStripMenuItem menuRemoveSelected;
        private ToolStripMenuItem menuRemoveAll;
        private ToolStripMenuItem menuSelect;
        private ToolStripMenuItem menuDeselect;
        private ToolStripMenuItem menuSelectAll;
        private ToolStripMenuItem menuDeselectAll;
        private ToolStripSeparator menuSeparator2;
        private ToolStripMenuItem menuHideUI;
        private ToolStripMenuItem menuHideUICurrent;
        private ToolStripMenuItem menuHideUISelected;
        private ToolStripMenuItem menuHideUIAll;
        private ToolStripSeparator menuSeparator3;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.contextMenu = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.menuConnect = new System.Windows.Forms.ToolStripMenuItem();
            this.menuConnectCurrent = new System.Windows.Forms.ToolStripMenuItem();
            this.menuConnectSelected = new System.Windows.Forms.ToolStripMenuItem();
            this.menuConnectAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDisconnect = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDisconnectCurrent = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDisconnectSelected = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDisconnectAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuSeparator1 = new System.Windows.Forms.ToolStripSeparator();
            this.menuShowUI = new System.Windows.Forms.ToolStripMenuItem();
            this.menuShowUICurrent = new System.Windows.Forms.ToolStripMenuItem();
            this.menuShowUISelected = new System.Windows.Forms.ToolStripMenuItem();
            this.menuShowUIAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuHideUI = new System.Windows.Forms.ToolStripMenuItem();
            this.menuHideUICurrent = new System.Windows.Forms.ToolStripMenuItem();
            this.menuHideUISelected = new System.Windows.Forms.ToolStripMenuItem();
            this.menuHideUIAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuSeparator2 = new System.Windows.Forms.ToolStripSeparator();
            this.menuSelect = new System.Windows.Forms.ToolStripMenuItem();
            this.menuSelectAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuSeparator3 = new System.Windows.Forms.ToolStripSeparator();
            this.menuRemove = new System.Windows.Forms.ToolStripMenuItem();
            this.menuRemoveCurrent = new System.Windows.Forms.ToolStripMenuItem();
            this.menuRemoveSelected = new System.Windows.Forms.ToolStripMenuItem();
            this.menuRemoveAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDeselectAll = new System.Windows.Forms.ToolStripMenuItem();
            this.menuDeselect = new System.Windows.Forms.ToolStripMenuItem();
            this.contextMenu.SuspendLayout();
            this.SuspendLayout();
            // 
            // contextMenu2
            // 
            this.contextMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuConnect,
            this.menuDisconnect,
            this.menuSeparator1,
            this.menuShowUI,
            this.menuHideUI,
            this.menuSeparator2,
            this.menuSelect,
            this.menuDeselect,
            this.menuSelectAll,
            this.menuDeselectAll,
            this.menuSeparator3,
            this.menuRemove});
            this.contextMenu.Name = "contextMenu";
            this.contextMenu.Size = new System.Drawing.Size(141, 198);
            this.contextMenu.Opening += new System.ComponentModel.CancelEventHandler(this.ContextMenu_Opening);
            // 
            // menuConnect
            // 
            this.menuConnect.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuConnectCurrent,
            this.menuConnectSelected,
            this.menuConnectAll});
            this.menuConnect.Image = global::fblock.manager.Properties.Resources.FBlockConnect;
            this.menuConnect.Name = "menuConnect";
            this.menuConnect.Size = new System.Drawing.Size(140, 22);
            this.menuConnect.Text = "Connect";
            // 
            // menuConnectCurrent
            // 
            this.menuConnectCurrent.Image = global::fblock.manager.Properties.Resources.FormSingle;
            this.menuConnectCurrent.Name = "menuConnectCurrent";
            this.menuConnectCurrent.Size = new System.Drawing.Size(126, 22);
            this.menuConnectCurrent.Text = "Current";
            this.menuConnectCurrent.Click += new System.EventHandler(this.MenuConnect_Click);
            // 
            // menuConnectSelected
            // 
            this.menuConnectSelected.Image = global::fblock.manager.Properties.Resources.FormDouble;
            this.menuConnectSelected.Name = "menuConnectSelected";
            this.menuConnectSelected.Size = new System.Drawing.Size(126, 22);
            this.menuConnectSelected.Text = "Selected";
            this.menuConnectSelected.Click += new System.EventHandler(this.MenuConnect_Click);
            // 
            // menuConnectAll
            // 
            this.menuConnectAll.Image = global::fblock.manager.Properties.Resources.FormCascade;
            this.menuConnectAll.Name = "menuConnectAll";
            this.menuConnectAll.Size = new System.Drawing.Size(126, 22);
            this.menuConnectAll.Text = "All";
            this.menuConnectAll.Click += new System.EventHandler(this.MenuConnect_Click);
            // 
            // menuDisconnect
            // 
            this.menuDisconnect.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuDisconnectCurrent,
            this.menuDisconnectSelected,
            this.menuDisconnectAll});
            this.menuDisconnect.Image = global::fblock.manager.Properties.Resources.FBlockDisconnect;
            this.menuDisconnect.Name = "menuDisconnect";
            this.menuDisconnect.Size = new System.Drawing.Size(140, 22);
            this.menuDisconnect.Text = "Disconnect";
            // 
            // menuDisconnectCurrent
            // 
            this.menuDisconnectCurrent.Image = global::fblock.manager.Properties.Resources.FormSingle;
            this.menuDisconnectCurrent.Name = "menuDisconnectCurrent";
            this.menuDisconnectCurrent.Size = new System.Drawing.Size(126, 22);
            this.menuDisconnectCurrent.Text = "Current";
            this.menuDisconnectCurrent.Click += new System.EventHandler(this.MenuDisconnect_Click);
            // 
            // menuDisconnectSelected
            // 
            this.menuDisconnectSelected.Image = global::fblock.manager.Properties.Resources.FormDouble;
            this.menuDisconnectSelected.Name = "menuDisconnectSelected";
            this.menuDisconnectSelected.Size = new System.Drawing.Size(126, 22);
            this.menuDisconnectSelected.Text = "Selected";
            this.menuDisconnectSelected.Click += new System.EventHandler(this.MenuDisconnect_Click);
            // 
            // menuDisconnectAll
            // 
            this.menuDisconnectAll.Image = global::fblock.manager.Properties.Resources.FormCascade;
            this.menuDisconnectAll.Name = "menuDisconnectAll";
            this.menuDisconnectAll.Size = new System.Drawing.Size(126, 22);
            this.menuDisconnectAll.Text = "All";
            this.menuDisconnectAll.Click += new System.EventHandler(this.MenuDisconnect_Click);
            // 
            // menuSeparator1
            // 
            this.menuSeparator1.Name = "menuSeparator1";
            this.menuSeparator1.Size = new System.Drawing.Size(137, 6);
            // 
            // menuShowUI
            // 
            this.menuShowUI.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuShowUICurrent,
            this.menuShowUISelected,
            this.menuShowUIAll});
            this.menuShowUI.Image = global::fblock.manager.Properties.Resources.FBlockShowUI;
            this.menuShowUI.Name = "menuShowUI";
            this.menuShowUI.Size = new System.Drawing.Size(140, 22);
            this.menuShowUI.Text = "Show UI";
            // 
            // menuShowUICurrent
            // 
            this.menuShowUICurrent.Image = global::fblock.manager.Properties.Resources.FormSingle;
            this.menuShowUICurrent.Name = "menuShowUICurrent";
            this.menuShowUICurrent.Size = new System.Drawing.Size(126, 22);
            this.menuShowUICurrent.Text = "Current";
            this.menuShowUICurrent.Click += new System.EventHandler(this.MenuShowUI_Click);
            // 
            // menuShowUISelected
            // 
            this.menuShowUISelected.Image = global::fblock.manager.Properties.Resources.FormDouble;
            this.menuShowUISelected.Name = "menuShowUISelected";
            this.menuShowUISelected.Size = new System.Drawing.Size(126, 22);
            this.menuShowUISelected.Text = "Selected";
            this.menuShowUISelected.Click += new System.EventHandler(this.MenuShowUI_Click);
            // 
            // menuShowUIAll
            // 
            this.menuShowUIAll.Image = global::fblock.manager.Properties.Resources.FormCascade;
            this.menuShowUIAll.Name = "menuShowUIAll";
            this.menuShowUIAll.Size = new System.Drawing.Size(126, 22);
            this.menuShowUIAll.Text = "All";
            this.menuShowUIAll.Click += new System.EventHandler(this.MenuShowUI_Click);
            // 
            // menuHideUI
            // 
            this.menuHideUI.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuHideUICurrent,
            this.menuHideUISelected,
            this.menuHideUIAll});
            this.menuHideUI.Image = global::fblock.manager.Properties.Resources.FBlockHideUI;
            this.menuHideUI.Name = "menuHideUI";
            this.menuHideUI.Size = new System.Drawing.Size(140, 22);
            this.menuHideUI.Text = "Hide UI";
            // 
            // menuHideUICurrent
            // 
            this.menuHideUICurrent.Image = global::fblock.manager.Properties.Resources.FormSingle;
            this.menuHideUICurrent.Name = "menuHideUICurrent";
            this.menuHideUICurrent.Size = new System.Drawing.Size(126, 22);
            this.menuHideUICurrent.Text = "Current";
            this.menuHideUICurrent.Click += new System.EventHandler(this.MenuHideUI_Click);
            // 
            // menuHideUISelected
            // 
            this.menuHideUISelected.Image = global::fblock.manager.Properties.Resources.FormDouble;
            this.menuHideUISelected.Name = "menuHideUISelected";
            this.menuHideUISelected.Size = new System.Drawing.Size(126, 22);
            this.menuHideUISelected.Text = "Selected";
            this.menuHideUISelected.Click += new System.EventHandler(this.MenuHideUI_Click);
            // 
            // menuHideUIAll
            // 
            this.menuHideUIAll.Image = global::fblock.manager.Properties.Resources.FormCascade;
            this.menuHideUIAll.Name = "menuHideUIAll";
            this.menuHideUIAll.Size = new System.Drawing.Size(126, 22);
            this.menuHideUIAll.Text = "All";
            this.menuHideUIAll.Click += new System.EventHandler(this.MenuHideUI_Click);
            // 
            // menuSeparator2
            // 
            this.menuSeparator2.Name = "menuSeparator2";
            this.menuSeparator2.Size = new System.Drawing.Size(137, 6);
            // 
            // menuSelect
            // 
            this.menuSelect.Image = global::fblock.manager.Properties.Resources.TickSingle;
            this.menuSelect.Name = "menuSelect";
            this.menuSelect.Size = new System.Drawing.Size(140, 22);
            this.menuSelect.Text = "Select";
            this.menuSelect.Click += new System.EventHandler(this.MenuSelect_Click);
            // 
            // menuDeselect
            // 
            this.menuDeselect.Image = global::fblock.manager.Properties.Resources.TickSingleMinus;
            this.menuDeselect.Name = "menuDeselect";
            this.menuDeselect.Size = new System.Drawing.Size(140, 22);
            this.menuDeselect.Text = "Deselect";
            this.menuDeselect.Click += new System.EventHandler(this.MenuDeselect_Click);
            // 
            // menuSelectAll
            // 
            this.menuSelectAll.Image = global::fblock.manager.Properties.Resources.TickDouble;
            this.menuSelectAll.Name = "menuSelectAll";
            this.menuSelectAll.Size = new System.Drawing.Size(140, 22);
            this.menuSelectAll.Text = "Select All";
            this.menuSelectAll.Click += new System.EventHandler(this.MenuSelectAll_Click);
            // 
            // menuDeselectAll
            // 
            this.menuDeselectAll.Image = global::fblock.manager.Properties.Resources.TickDoubleMinus;
            this.menuDeselectAll.Name = "menuDeselectAll";
            this.menuDeselectAll.Size = new System.Drawing.Size(140, 22);
            this.menuDeselectAll.Text = "Deselect All";
            this.menuDeselectAll.Click += new System.EventHandler(this.MenuDeselectAll_Click);
            // 
            // menuSeparator3
            // 
            this.menuSeparator3.Name = "menuSeparator3";
            this.menuSeparator3.Size = new System.Drawing.Size(137, 6);
            // 
            // menuRemove
            // 
            this.menuRemove.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.menuRemoveCurrent,
            this.menuRemoveSelected,
            this.menuRemoveAll});
            this.menuRemove.Image = global::fblock.manager.Properties.Resources.Delete;
            this.menuRemove.Name = "menuRemove";
            this.menuRemove.Size = new System.Drawing.Size(140, 22);
            this.menuRemove.Text = "Remove";
            // 
            // menuRemoveCurrent
            // 
            this.menuRemoveCurrent.Image = global::fblock.manager.Properties.Resources.FormSingle;
            this.menuRemoveCurrent.Name = "menuRemoveCurrent";
            this.menuRemoveCurrent.Size = new System.Drawing.Size(126, 22);
            this.menuRemoveCurrent.Text = "Current";
            this.menuRemoveCurrent.Click += new System.EventHandler(this.MenuRemove_Click);
            // 
            // menuRemoveSelected
            // 
            this.menuRemoveSelected.Image = global::fblock.manager.Properties.Resources.FormDouble;
            this.menuRemoveSelected.Name = "menuRemoveSelected";
            this.menuRemoveSelected.Size = new System.Drawing.Size(126, 22);
            this.menuRemoveSelected.Text = "Selected";
            this.menuRemoveSelected.Click += new System.EventHandler(this.MenuRemove_Click);
            // 
            // menuRemoveAll
            // 
            this.menuRemoveAll.Image = global::fblock.manager.Properties.Resources.FormCascade;
            this.menuRemoveAll.Name = "menuRemoveAll";
            this.menuRemoveAll.Size = new System.Drawing.Size(126, 22);
            this.menuRemoveAll.Text = "All";
            this.menuRemoveAll.Click += new System.EventHandler(this.MenuRemove_Click);
            // 
            // FBlockTreeView
            // 
            this.AllowDrop = true;
            this.ContextMenuStrip = this.contextMenu;
            this.HideSelection = false;
            this.LineColor = System.Drawing.Color.Black;
            this.NodeMouseDoubleClick += new System.Windows.Forms.TreeNodeMouseClickEventHandler(this.FBlockTreeView_NodeMouseDoubleClick);
            this.NodeMouseClick += new System.Windows.Forms.TreeNodeMouseClickEventHandler(this.FBlockTreeView_NodeMouseClick);
            this.ItemDrag += new ItemDragEventHandler(this.FBlockTreeView_ItemDrag);
            this.DragEnter += new DragEventHandler(this.FBlockTreeView_DragEnter);
            this.DragOver += new DragEventHandler(this.FBlockTreeView_DragOver);
            this.DragDrop += new DragEventHandler(this.FBlockTreeView_DragDrop);
            this.contextMenu.ResumeLayout(false);
            this.ResumeLayout(false);
        }

        private void PostInitializeComponent()
        {
            this.ImageList = new ImageList();
            this.ImageList.Images.Add(IconKeys.FBlockRoot.ToString(), global::fblock.manager.Properties.Resources.FBlockRoot);
            this.ImageList.Images.Add(IconKeys.FBlockShowUI.ToString(), global::fblock.manager.Properties.Resources.FBlockShowUI);
            this.ImageList.Images.Add(IconKeys.FBlockShowUIPlay.ToString(), global::fblock.manager.Properties.Resources.FBlockShowUIPlay);
            this.ImageList.Images.Add(IconKeys.FBlockHideUI.ToString(), global::fblock.manager.Properties.Resources.FBlockHideUI);
            this.ImageList.Images.Add(IconKeys.FBlockHideUIPlay.ToString(), global::fblock.manager.Properties.Resources.FBlockHideUIPlay);

            this.contextMenu.RenderMode = ToolStripRenderMode.System;
            this.contextMenu.Renderer = new ToolStripProfessionalRenderer(new CustomColorTable());
        }

        #endregion // FBlockTreeView component setup
    }
}

// EOF