﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;

namespace fblock.manager
{
    public class Logging
    {
        private enum LabelType { MESSAGE, WARNING, ERROR, DEBUG };
        private static Logging theInstance = null;
        private TextWriter logger = null;
        private string filename = null;
        private bool ensure = false;
        private bool overwrite = false;
        private bool timestamp = false;
        private bool labels = false;

        #region Logging construction

        private Logging() : base() { }

        public static Logging Instance
        {
            get
            {
                if (Logging.theInstance == null)
                {
                    Logging.theInstance = new Logging();
                }
                return Logging.theInstance;
            }
        }

        #endregion // Logging construction

        #region Logging properties

        public string Filename
        {
            get { return String.IsNullOrEmpty(this.filename) ? "" : this.filename; }
        }

        public bool Ensure
        {
            get { return this.ensure; }
            set { this.ensure = value; }
        }

        public bool Overwrite
        {
            get { return this.overwrite; }
            set { this.overwrite = value; }
        }

        public bool Timestamp
        {
            get { return this.timestamp; }
            set { this.timestamp = value; }
        }

        public bool Labels
        {
            get { return this.labels; }
            set { this.labels = value; }
        }

        #endregion // Logging properties

        #region Logging access

        public void Setup(ApplicationSettings.LoggingSettings settings)
        {
            this.filename = settings.Filename;
            this.ensure = settings.Ensure;
            this.overwrite = settings.Overwrite;
            this.timestamp = settings.Timestamp;
            this.labels = settings.Labels;
        }

        public bool Open()
        {
            return this.Open(this.Filename);
        }

        public bool Open(string filename)
        {
            if (this.logger == null)
            {
                this.filename = filename;

                if (File.Exists(this.Filename))
                {
                    if (this.Overwrite)
                    {
                        try
                        {
                            File.Delete(this.Filename);
                        }
                        catch (Exception exception)
                        {
                            Logging.ReportWarning("Deleting file [" + Path.GetFileName(this.Filename) + "] failed.", "Open", exception);
                            return false;
                        }
                    }
                    else
                    {
                        try
                        {
                            File.Move(this.Filename, this.Destination(this.Filename, DateTime.Now));
                        }
                        catch (Exception exception)
                        {
                            Logging.ReportWarning("Renaming file [" + Path.GetFileName(this.Filename) + "] failed.", "Open", exception);
                            return false;
                        }
                    }
                }
                else if (this.Ensure)
                {
                    try
                    {
                        string folder = Path.GetDirectoryName(this.Filename);
                        if (!String.IsNullOrEmpty(folder))
                        {
                            Directory.CreateDirectory(folder);
                        }
                    }
                    catch (Exception exception)
                    {
                        Logging.ReportError("Creating directory [" + Path.GetDirectoryName(this.Filename) + "] failed.", "Open", exception);
                        return false;
                    }
                }

                try
                {
                    this.logger = File.CreateText(this.Filename);
                    this.SelfEvent("Logging startet");
                    return true;
                }
                catch (Exception exception)
                {
                    Logging.ReportError("Creating log-file [" + Path.GetFileName(this.Filename) + "] failed.", "Open", exception);
                    return false;
                }
            }
            else
            {
                Logging.ReportWarning("Logging already opend.", "Open");
                return false;
            }
        }

        public bool Reopen()
        {
            this.Close();
            return this.Open();
        }

        public void Message(string message)
        {
            if (this.IsLogging())
            {
                this.Write(this.Format(message, LabelType.MESSAGE));
            }
        }

        public void Warning(string message)
        {
            if (this.IsLogging())
            {
                this.Write(this.Format(message, LabelType.WARNING));
            }
        }

        public void Error(string message)
        {
            if (this.IsLogging())
            {
                this.Write(this.Format(message, LabelType.ERROR));
            }
        }

        public void Debug(string message)
        {
#if DEBUG
            if (this.IsLogging())
            {
                this.Write(this.Format(message, LabelType.DEBUG));
            }
#endif // DEBUG
        }

        public void Close()
        {
            this.SelfEvent("Logging stopped");

            if (this.logger != null)
            {
                this.logger.Close();
                this.logger = null;
            }
        }

        #endregion // Logging access

        #region Private implementation

        private bool IsLogging()
        {
            if (this.logger != null)
            {
                // Check if logging is enabled on demand!
                return ApplicationSettings.Instance.Logging.Enabled;
            }
            else
            {
                return false;
            }
        }

        private void Write(string message)
        {
            try
            {
                if (!String.IsNullOrEmpty(message))
                {
                    this.logger.WriteLine(message);
                    this.logger.Flush();
                }
            }
            catch (Exception /*exception*/)
            {
                // Do not call any report sink function because of possibly dead locks!
            }
        }

        private string Format(string message, LabelType type)
        {
            string result = "";

            if (!String.IsNullOrEmpty(message))
            {
                if (this.Timestamp)
                {
                    DateTime date = DateTime.Now;
                    result += String.Format("{0:D4}-{1:D2}-{2:D2} {3:D2}:{4:D2}:{5:D2}.{6:D3} ",
                        date.Year,
                        date.Month,
                        date.Day,
                        date.Hour,
                        date.Minute,
                        date.Second,
                        date.Millisecond);
                }

                if (this.Labels)
                {
                    result += String.Format(" {0,-9} ", ("[" + type.ToString()) + "]");
                }

                result += message;
            }

            return result.Trim();
        }

        private string Destination(string filename, DateTime date)
        {
            string folder = Path.GetDirectoryName(filename);
            string file = Path.GetFileNameWithoutExtension(filename);
            string extension = Path.GetExtension(filename);
            string label = String.Format("_{0:D4}-{1:D2}-{2:D2}_{3:D2}.{4:D2}.{5:D2}.{6:D3}", date.Year, date.Month, date.Day, date.Hour, date.Minute, date.Second, date.Millisecond);

            if (String.IsNullOrEmpty(folder))
            {
                folder = Directory.GetCurrentDirectory();
            }
            folder += Path.DirectorySeparatorChar;

            return folder + file + label + extension;
        }

        private void SelfEvent(string message)
        {
            string output = "";
            if (this.Timestamp)
            {
                output = message;
            }
            else
            {
                DateTime date = DateTime.Now;
                output = String.Format("{0} at {1:D4}-{2:D2}-{3:D2} {4:D2}:{5:D2}:{6:D2}.{7:D3}", message, date.Year, date.Month, date.Day, date.Hour, date.Minute, date.Second, date.Millisecond);
            }
            this.Message(output);
        }

        #region Main Form Report Sink access

        private static void ReportMessage(string description)
        {
            Logging.ReportMessage(description, null, null);
        }

        private static void ReportMessage(string description, string category)
        {
            Logging.ReportMessage(description, category, null);
        }

        private static void ReportMessage(string description, object additional)
        {
            Logging.ReportMessage(description, null, additional);
        }

        private static void ReportMessage(string description, string category, object additional)
        {
            if (Program.Instance.MainForm != null && Program.Instance.MainForm.ReportSink != null)
            {
                Program.Instance.MainForm.ReportSink.Message(description, "Logging", category, additional);
            }
        }

        private static void ReportWarning(string description)
        {
            Logging.ReportWarning(description, null, null);
        }

        private static void ReportWarning(string description, string category)
        {
            Logging.ReportWarning(description, category, null);
        }

        private static void ReportWarning(string description, object additional)
        {
            Logging.ReportWarning(description, null, additional);
        }

        private static void ReportWarning(string description, string category, object additional)
        {
            if (Program.Instance.MainForm != null && Program.Instance.MainForm.ReportSink != null)
            {
                Program.Instance.MainForm.ReportSink.Warning(description, "Logging", category, additional);
            }
        }

        private static void ReportError(string description)
        {
            Logging.ReportError(description, null, null);
        }

        private static void ReportError(string description, string category)
        {
            Logging.ReportError(description, category, null);
        }

        private static void ReportError(string description, object additional)
        {
            Logging.ReportError(description, null, additional);
        }

        private static void ReportError(string description, string category, object additional)
        {
            if (Program.Instance.MainForm != null && Program.Instance.MainForm.ReportSink != null)
            {
                Program.Instance.MainForm.ReportSink.Error(description, "Logging", category, additional);
            }
        }

        #endregion // Main Form Report Sink access

        #endregion // Private implementation
    }
}

// EOF