﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace fblock.manager
{
    partial class LoggingDialog : Form
    {
        private ApplicationSettings.LoggingSettings settings = null;

        public LoggingDialog(ApplicationSettings.LoggingSettings settings)
            : base()
        {
            System.Diagnostics.Debug.Assert(settings != null);

            this.settings = new ApplicationSettings.LoggingSettings(settings);

            this.InitializeComponent();
            this.PostInitializeComponent();
        }

        public LoggingDialog(ApplicationSettings.LoggingSettings settings, bool useBackButton)
            : this(settings)
        {
            if (useBackButton == true)
            {
                this.btnCancel.Hide();
                this.btnSave.Location = this.btnCancel.Location;
                this.btnSave.Text = "&Back";
            }
        }

        public ApplicationSettings.LoggingSettings Settings
        {
            get { return this.settings; }
        }

        private void PostInitializeComponent()
        {
            this.SetFilename(this.settings.Filename);
            this.chkLogEnabled.Checked = this.settings.Enabled;
            this.grpLogFileSettings.Enabled = this.chkLogEnabled.Checked;
            this.chkEnsureFolder.Checked = this.settings.Ensure;
            this.chkOverwriteLogfile.Checked = this.settings.Overwrite;
            this.chkUseTimestamp.Checked = this.settings.Timestamp;
            this.chkUseLabels.Checked = this.settings.Labels;
        }

        #region Control handling

        private void btnSave_Click(object sender, EventArgs args)
        {
            try
            {
                this.settings.Enabled = this.chkLogEnabled.Checked;
                this.settings.Ensure = this.chkEnsureFolder.Checked;
                this.settings.Overwrite = this.chkOverwriteLogfile.Checked;
                this.settings.Filename = this.GetFilename();

                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            catch (InvalidDataException exception)
            {
                MessageBox.Show(exception.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                this.txtLogFilename.Focus();
            }
        }

        private void btnLogFilename_Click(object sender, EventArgs args)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "Log files (*.txt,*.log)|*.txt;*.log|All files (*.*)|*.*";
            dialog.Multiselect = false;
            dialog.CheckFileExists = false;
            dialog.CheckPathExists = false;
            dialog.RestoreDirectory = false;
            dialog.FileName = this.txtLogFilename.Text;
            dialog.InitialDirectory = String.IsNullOrEmpty((string)this.txtLogFilename.Tag) ? ApplicationSettings.Location : (string)this.txtLogFilename.Tag;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                this.SetFilename(dialog.FileName);
            }
        }

        private void chkLogEnabled_CheckedChanged(object sender, EventArgs args)
        {
            this.grpLogFileSettings.Enabled = this.chkLogEnabled.Checked;
        }

        private void txtLogFilename_MouseHover(object sender, EventArgs args)
        {
            if (!String.IsNullOrEmpty((string)this.txtLogFilename.Tag))
            {
                this.tipGeneral.SetToolTip(this.txtLogFilename, "Folder: " + (string)this.txtLogFilename.Tag);
            }
            else
            {
                this.tipGeneral.SetToolTip(this.txtLogFilename, "Log file name and path.");
            }
        }

        private void chkUseTimestamp_MouseHover(object sender, EventArgs args)
        {
            DateTime date = DateTime.Now;
            this.tipGeneral.SetToolTip(this.chkUseTimestamp, String.Format("Sample: {0:D4}-{1:D2}-{2:D2} {3:D2}:{4:D2}:{5:D2}.{6:D3} ...", date.Year, date.Month, date.Day, date.Hour, date.Minute, date.Second, date.Millisecond));
        }

        #endregion // Control handling

        #region Filename handling

        private string GetFilename()
        {
            if (!String.IsNullOrEmpty(this.txtLogFilename.Text))
            {
                return (string)this.txtLogFilename.Tag + this.txtLogFilename.Text;
            }
            else
            {
                if (!this.chkLogEnabled.Checked)
                {
                    return (string)this.txtLogFilename.Tag;
                }
                else
                {
                    throw new InvalidDataException("Please provide an appropriated file name!");
                }
            }
        }

        private void SetFilename(string filename)
        {
            if (!String.IsNullOrEmpty(filename))
            {
                string fullpath = Path.GetFullPath(filename);

                this.txtLogFilename.Tag = Path.GetDirectoryName(fullpath) + Path.DirectorySeparatorChar;
                this.txtLogFilename.Text = Path.GetFileName(fullpath);
            }
            else
            {
                this.txtLogFilename.Tag = "";
                this.txtLogFilename.Text = "";
            }
        }

        #endregion // Filename handling
    }
}

// EOF