﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.Specialized;

namespace fblock.manager
{
    public partial class MainForm : Form
    {
        private bool dirty = false;
        private string mainTitle = "";
        private string cmdProject = null;
        private bool cmdHide = false;
        private FormWindowState lastState;

        public MainForm()
        {
            this.InitializeComponent();
            this.lastState = this.WindowState;
            this.mainTitle = this.Text; // Save initial window title.
            this.PostInitializeComponent();
        }

        public void Redraw()
        {
            this.Invalidate(this.Bounds, true);
            this.Update();
            Application.DoEvents();
        }

        public bool Dirty
        {
            get { return this.dirty; }
            set { this.dirty = value; }
        }

        public int MainSplitDistance
        {
            get { return this.mainSplitContainer.SplitterDistance; }
            set
            {
                try
                {
                    this.mainSplitContainer.SplitterDistance = value;
                }
                catch (InvalidOperationException exception)
                {
                    // REMARK: This exception occurs if the form was maximized, then minimized and closed afterwards!
                    this.reportSinkPanel.Warning("Adjusting splitter distance (main panel) failed!", "FBlockManager", "Settings", exception);
                }
            }
        }

        public int TreeSplitDistance
        {
            get { return this.treeSplitContainer.SplitterDistance; }
            set
            {
                try
                {
                    this.treeSplitContainer.SplitterDistance = value;
                }
                catch (InvalidOperationException exception)
                {
                    // REMARK: This exception occurs if the form was maximized, then minimized and closed afterwards!
                    this.reportSinkPanel.Warning("Adjusting splitter distance (tree panel) failed!", "FBlockManager", "Settings", exception);
                }
            }
        }

        public string CommandLineProject
        {
            get { return this.cmdProject; }
            set { this.cmdProject = value; }
        }

        public bool CommandLineHide
        {
            get { return this.cmdHide; }
            set { this.cmdHide = value; }
        }

        public ReportSink ReportSink
        {
            get { return this.reportSinkPanel; }
        }

        public void GetInfoPaneColor(out Color foreground, out Color background)
        {
            foreground = this.fblockTextBox.ForeColor;
            background = this.fblockTextBox.BackColor;
        }

        public void SetInfoPaneColor(Color foreground, Color background)
        {
            this.fblockTextBox.ForeColor = foreground;
            this.fblockTextBox.BackColor = background;
            this.Redraw();
        }

        public void GetTreeViewColor(out Color foreground, out Color background)
        {
            foreground = this.fblockTreeView.ForeColor;
            background = this.fblockTreeView.BackColor;
        }

        public void SetTreeViewColor(Color foreground, Color background)
        {
            this.fblockTreeView.ForeColor = foreground;
            this.fblockTreeView.BackColor = background;
            this.Redraw();
        }

        public void GetReportSinkColor(out Color foreground, out Color background)
        {
            foreground = this.reportSinkPanel.ReportListView.ForeColor;
            background = this.reportSinkPanel.ReportListView.BackColor;
        }

        public void SetReportSinkColor(Color foreground, Color background)
        {
            this.reportSinkPanel.ReportListView.ForeColor = foreground;
            this.reportSinkPanel.ReportListView.BackColor = background;
            this.Redraw();
        }

        #region Menu event definition section

        private void MenuFileProjectNew_Click(object sender, EventArgs args)
        {
            // TODO: Implement only if needed (see comment in PostInitializeComponent())
        }

        private void MenuFileProjectOpen_Click(object sender, EventArgs args)
        {
            this.Redraw();
            this.OpenProject(null);
        }

        private void MenuFileProjectSave_Click(object sender, EventArgs args)
        {
            this.Redraw();
            if (this.fblockTreeView.SaveProject())
            {
                this.Dirty = false;
            }
        }

        private void MenuFileProjectSaveAs_Click(object sender, EventArgs args)
        {
            this.Redraw();
            if (this.fblockTreeView.SaveProjectAs())
            {
                this.Dirty = false;
            }
        }

        private void MenuFileProjectClose_Click(object sender, EventArgs args)
        {
            this.Redraw();
            if (this.CanCloseProject("Do you want to save changes before close this project?"))
            {
                this.fblockTreeView.CloseProject();
                this.Dirty = false;
            }
        }

        private void MenuFileImportFile_Click(object sender, EventArgs args)
        {
            this.Redraw();
            if (this.fblockTreeView.ImportFromFile())
            {
                this.Dirty = true;
            }
        }

        private void MenuFileImportFolder_Click(object sender, EventArgs args)
        {
            this.Redraw();
            if (this.fblockTreeView.ImportFromFolder())
            {
                this.Dirty = true;
            }
        }

        private void MenuFileExit_Click(object sender, EventArgs args)
        {
            this.Redraw();
            this.Close();
        }

        private void MainMenuFBlock_DropDownOpening(object sender, EventArgs args)
        {
            this.mainMenuFBlocks.DropDown = this.fblockTreeView.ContextMenuStrip;
        }

        private void MainMenuReport_DropDownOpening(object sender, EventArgs args)
        {
            this.mainMenuReport.DropDown = this.reportSinkPanel.ContextMenuStrip;
        }

        private void MainMenuFile_DropDownOpened(object sender, EventArgs args)
        {
            this.menuFileProjectSave.Enabled = this.Dirty;
            this.menuFileProjectSaveAs.Enabled = this.fblockTreeView.SaveAsEnabled;
            this.menuFileProjectClose.Enabled = this.Dirty || this.fblockTreeView.CloseEnabled;

            // Initialize "Recent Projects" with currently available MRUs
            this.menuFileRecentProjects.DropDownItems.Clear();
            foreach (string fullname in ApplicationSettings.Instance.MRUProjects)
            {
                // Try to add a new MRU item.
                ToolStripItem item = this.CreateMRUProjectsEntry(fullname);
                if (item != null)
                {
                    // Add new item.
                    this.menuFileRecentProjects.DropDownItems.Add(item);
                }
            }
            // Enabel/Disable "Recent Projects" depending on available items.
            if (this.menuFileRecentProjects.DropDownItems.Count > 0)
            {
                this.menuFileRecentProjects.Enabled = true;
            }
            else
            {
                this.menuFileRecentProjects.Enabled = false;
                this.menuFileRecentProjects.DropDownItems.Add("<nothing>").Enabled = false;
            }
        }

        private void MainMenuRecentProjectsItem_Click(object sender, EventArgs args)
        {
            this.Redraw();
            this.OpenProject(((ToolStripItem)sender).Name);
        }

        private void MainMenuHelpAbout_Click(object sender, EventArgs args)
        {
            AboutDialog dialog = new AboutDialog();
            dialog.ShowDialog();
            this.Redraw();
        }

        private void MainMenuHelpHelp_Click(object sender, EventArgs args)
        {
            ShortHelpDialog dialog = new ShortHelpDialog();
            dialog.ShowDialog();
            this.Redraw();
        }

        private void MainMenuTools_DropDownOpening(object sender, EventArgs args)
        {
            this.mainMenuToolsOptions.Enabled = this.fblockTreeView.ProjectSettingsEnabled;
        }

        private void MainMenuToolsSettings_Click(object sender, EventArgs args)
        {
            ApplicationSettingsDialog dialog = new ApplicationSettingsDialog(this);
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                ApplicationSettings.UseFromCopy(dialog.Settings);

                // Inform about changed settings.
                this.fblockTreeView.ApplicationSettingsChanged();
                this.reportSinkPanel.ApplicationSettingsChanged();
            }
            this.Redraw();
        }

        private void MainMenuToolsOptions_Click(object sender, EventArgs args)
        {
            ProjectSettingsDialog dialog = new ProjectSettingsDialog(this, this.fblockTreeView.CurrentProjectSettings);
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                // Nothing else to do!
            }
            this.Redraw();
        }

        #endregion // Menu event definition section

        #region Main form event definition section

        private void MainForm_Shown(object sender, EventArgs args)
        {
            this.MainSplitDistance = ApplicationSettings.Instance.MainSplitDistance;
            this.TreeSplitDistance = ApplicationSettings.Instance.TreeSplitDistance;
            this.Redraw();

            // Command line parameter overwrites auto load settings!
            if (!String.IsNullOrEmpty(this.cmdProject))
            {
                if (File.Exists(this.cmdProject))
                {
                    this.OpenProject(this.cmdProject);
                    this.fblockTreeView.AutoConnect();
                }
                else
                {
                    this.reportSinkPanel.Warning("Project file not found!", "FBlockManager", "Open", (!String.IsNullOrEmpty(this.cmdProject) ? this.cmdProject : "<empty>"));
                }
            }
            else if (ApplicationSettings.Instance.AutoOpenLastProject)
            {
                string fullname = ApplicationSettings.Instance.LastProject;
                if (File.Exists(fullname))
                {
                    this.OpenProject(fullname);
                    this.fblockTreeView.AutoConnect();
                }
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs args)
        {
            args.Cancel = !this.CanCloseProject("Do you want to save changes before exit the program?");
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs args)
        {
            try
            {
                this.fblockTreeView.CloseProject();
            }
            catch (Exception /*exception*/)
            {
            }
        }

        private void MainForm_Move(object sender, EventArgs args)
        {
            if (this.WindowState == System.Windows.Forms.FormWindowState.Normal)
            {
                ApplicationSettings.Instance.MainFormLocation = this.Location;
            }
        }

        private void MainForm_Resize(object sender, EventArgs args)
        {
            if (this.WindowState == System.Windows.Forms.FormWindowState.Normal)
            {
                ApplicationSettings.Instance.MainFormDimension = this.Size;
            }
        }

        private void MainForm_DragEnter(object sender, DragEventArgs args)
        {
            try
            {
                // Do nothing if the tree view uses drag&drop!
                if (this.fblockTreeView.IsDragging)
                {
                    return;
                }

                DataObject data = (DataObject)args.Data;
                if (!data.ContainsFileDropList())
                {
                    args.Effect = DragDropEffects.None;
                    return;
                }

                StringCollection files = data.GetFileDropList();
                if (files.Count > 1)
                {
                    args.Effect = DragDropEffects.None;
                    return;
                }

                if (Path.GetExtension(files[0]).ToLower() != ".fblock")
                {
                    args.Effect = DragDropEffects.None;
                    return;
                }

                args.Effect = DragDropEffects.Move;
            }
            catch (Exception)
            {
                args.Effect = DragDropEffects.None;
            }
        }

        private void MainForm_DragDrop(object sender, DragEventArgs args)
        {
            try
            {
                this.Redraw();
                // Get drop file name and open it.
                // Note, number of files is limited in DragEnter!
                this.OpenProject(((string[])args.Data.GetData("FileNameW"))[0]);
            }
            catch (Exception)
            {
            }
        }

        #endregion // Main form event definition section

        #region Tree view event definition section

        private void FBlockTreeView_AfterSelect(object sender, TreeViewEventArgs args)
        {
            try
            {
                this.fblockTextBox.SetFBlock((FBlockInvoker)args.Node.Tag);
            }
            catch (Exception /*exception*/)
            {
            }
        }

        private void FBlockTreeView_CurrentProjectChanged(object sender, EventArgs args)
        {
            if (!String.IsNullOrEmpty(this.fblockTreeView.CurrentProject))
            {
                this.Text = this.fblockTreeView.CurrentProject + " - " + this.mainTitle;
            }
            else
            {
                this.Text = this.mainTitle;
            }
        }

        #endregion // Tree view event definition section

        private ToolStripItem CreateMRUProjectsEntry(string fullname)
        {
            if (!String.IsNullOrEmpty(fullname))
            {
                // Try to find an already existing entry.
                if (this.menuFileRecentProjects.DropDownItems.Find(fullname, false).Length == 0)
                {
                    // Put to the front of the list!
                    ToolStripMenuItem result = new ToolStripMenuItem();

                    result.Text = Path.GetFileName(fullname);
                    result.Name = fullname;
                    result.Click += new EventHandler(this.MainMenuRecentProjectsItem_Click);

                    return result;
                }
            }

            return null;
        }

        private void OpenProject(string fullname)
        {
            if (this.CanCloseProject("Do you want to save changes before open another project?"))
            {
                if (String.IsNullOrEmpty(fullname))
                {
                    if (this.fblockTreeView.OpenProject())
                    {
                        this.Dirty = false;
                    }
                }
                else
                {
                    this.fblockTreeView.CloseProject();
                    this.fblockTreeView.OpenProject(fullname);
                    this.Dirty = false;
                }

                // Activate form!
                this.PutActivate();
            }
        }

        private bool CanCloseProject(string message)
        {
            if (this.Dirty)
            {
                switch (MessageBox.Show(message, this.Text, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                {
                    case DialogResult.Cancel:
                        // Cancel closing project!
                        return false;
                    case DialogResult.Yes:
                        // Do it this way to avoid duplicate code.
                        this.Redraw();
                        this.MenuFileProjectSave_Click(this.menuFileProjectSave, new EventArgs());
                        return true;
                    case DialogResult.No:
                        // Don't save changes and finish
                        return true;
                    default:
                        return false; // Cannot happen but avoids compiler error message!
                }
            }
            else
            {
                return true;
            }
        }

        private void PostInitializeComponent()
        {
            this.fblockTreeView.ReportSink = this.reportSinkPanel;
            this.fblockTreeView.AfterSelect += new TreeViewEventHandler(FBlockTreeView_AfterSelect);
            this.fblockTreeView.CurrentProjectChanged += new EventHandler<EventArgs>(FBlockTreeView_CurrentProjectChanged);

            this.mainMenuStrip.RenderMode = ToolStripRenderMode.System;
            this.mainMenuStrip.Renderer = new ToolStripProfessionalRenderer(new CustomColorTable());

            // TODO: Clarify if this menu item is really needed!
            this.mainMenuFile.DropDownItems.Remove(this.menuFileProjectNew);

            // Initialize that report sink.
            ShellLink.SetReportSink(this.reportSinkPanel);
        }

        private void PutActivate()
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                // Show form if invisible.
                this.WindowState = this.lastState;
                this.Show();
            }

            // Set form active!
            this.Activate();
        }

        private void PutShutdown(bool save)
        {
            // Current project will be saved if:
            // 1) Save changes is set to true (shall save)
            // 2) Last changes not saved yet (must save)
            // 3) A project file was already defined (can save)
            // Otherwise it is not possible to save 
            // a project automatically

            if (save && this.Dirty && this.fblockTreeView.SaveAsEnabled)
            {
                this.fblockTreeView.SaveProject();
            }
            this.Dirty = false;
            this.Redraw();
            this.Close();
        }

        protected override void OnResize(EventArgs args)
        {
            base.OnResize(args);

            if (this.lastState != this.WindowState && this.WindowState != FormWindowState.Minimized)
            {
                this.lastState = this.WindowState;
            }
        }

        protected override void WndProc(ref Message message)
        {
            if (message.Msg == InstanceAccessor.WM_APP_ALIVE)
            {
                // Just answer this request and return.
                message.Result = new IntPtr(InstanceAccessor.APP_ALIVE_RESULT);
            }
            else if (message.Msg == InstanceAccessor.WM_APP_FRONT)
            {
                // Activate form!
                this.PutActivate();

                // Report event.
                this.reportSinkPanel.Message("External application activation request received.", "FBlockManager", "External");

                // Answer this request and return.
                message.Result = new IntPtr(InstanceAccessor.APP_FRONT_RESULT);
            }
            else if (message.Msg == InstanceAccessor.WM_APP_CLOSE)
            {
                try
                {
                    // Try save current project.
                    this.PutShutdown(Convert.ToBoolean(message.LParam.ToInt32())); // LParam = save changes.

                    // Report event.
                    this.reportSinkPanel.Message("External application shutdown request received.", "FBlockManager", "External");

                    // Answer this request and return.
                    message.Result = new IntPtr(InstanceAccessor.APP_CLOSE_RESULT);
                }
                catch (Exception exception)
                {
                    this.reportSinkPanel.Message("External application shutdown request failed.", "FBlockManager", "External", exception);
                }
            }
            else
            {
                base.WndProc(ref message);
            }
        }
    }
}

// EOF