﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace fblock.manager
{
    public class Program : IReportSink
    {
        [STAThread]
        static void Main(string[] args)
        {
            CommandLineParser parser = new CommandLineParser(args);
            if (parser.Initialize)
            {
                Program.Instance.Setup(parser);
            }
            else if (parser.Shutdown)
            {
                InstanceAccessor.RequestShutdown(parser.NoSave);
            }
            else
            {
                if (Program.Instance.Initialize(parser))
                {
                    Program.Instance.Execute(parser);
                }
                Program.Instance.Cleanup();
            }
        }

        private static Program theInstance = null;

        private MainForm mainForm = null;

        private Program() : base() { }

        public static Program Instance
        {
            get
            {
                if (theInstance == null)
                {
                    theInstance = new Program();
                }
                return theInstance;
            }
        }

        public bool Initialize(CommandLineParser parser)
        {
            bool success = false;

            if (this.mainForm == null)
            {
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                try
                {
                    // Load application settings from default configuration.
                    ApplicationSettings.Instance.Deserialize();

                    // Check for application is already running 
                    // but only if singleton execution is set.
                    if (ApplicationSettings.Instance.Singleton && InstanceAccessor.IsRunning())
                    {
                        // Bring running application to front.
                        InstanceAccessor.RequestActivate();

                        // Finish this application instance.
                        return false;
                    }

                    // Overwrite logging settings with command line options.
                    if (parser.IsLogging)
                    {
                        ApplicationSettings.Instance.Logging.Enabled = true;
                        ApplicationSettings.Instance.Logging.Filename = parser.Logfile;
                        ApplicationSettings.Instance.Logging.Overwrite = parser.Overwrite;
                    }

                    // Initialize logging instance!
                    Logging.Instance.Setup(ApplicationSettings.Instance.Logging);

                    // Create application's main form.
                    this.mainForm = new MainForm();

                    // Start logging instance (ReportSink should be available)!
                    Logging.Instance.Open();

                    this.mainForm.Location = this.LocationToScreen(new Rectangle(ApplicationSettings.Instance.MainFormLocation, ApplicationSettings.Instance.MainFormDimension));
                    this.mainForm.Size = ApplicationSettings.Instance.MainFormDimension;
                    this.mainForm.WindowState = ApplicationSettings.Instance.MainFormWindowState;

                    this.mainForm.SetInfoPaneColor(
                        Color.FromArgb(ApplicationSettings.Instance.InfoPaneForeground),
                        Color.FromArgb(ApplicationSettings.Instance.InfoPaneBackground)
                    );

                    this.mainForm.SetTreeViewColor(
                        Color.FromArgb(ApplicationSettings.Instance.TreeViewForeground),
                        Color.FromArgb(ApplicationSettings.Instance.TreeViewBackground)
                    );

                    this.mainForm.SetReportSinkColor(
                        Color.FromArgb(ApplicationSettings.Instance.ReportSinkForeground),
                        Color.FromArgb(ApplicationSettings.Instance.ReportSinkBackground)
                    );

                    success = true;
                }
                catch (Exception exception)
                {
                    ErrorDialog dialog = new ErrorDialog(exception);
                    dialog.Caption += " (Initialize)";
                    dialog.ShowDialog();
                }
            }
            return success;
        }

        public bool Execute(CommandLineParser parser)
        {
            bool success = false;

            if (this.mainForm != null)
            {
                try
                {
                    // Save relevant command line settings.
                    this.mainForm.CommandLineProject = parser.InputFile;
                    this.mainForm.CommandLineHide = parser.Hide;

                    Application.Run(this.mainForm);

                    ApplicationSettings.Instance.MainFormWindowState = this.mainForm.WindowState;
                    if (this.mainForm.WindowState == FormWindowState.Normal)
                    {
                        ApplicationSettings.Instance.MainFormLocation = this.mainForm.Location;
                        ApplicationSettings.Instance.MainFormDimension = this.mainForm.Size;
                    }

                    ApplicationSettings.Instance.MainSplitDistance = this.mainForm.MainSplitDistance;
                    ApplicationSettings.Instance.TreeSplitDistance = this.mainForm.TreeSplitDistance;

                    // Save application settings into default configuration.
                    ApplicationSettings.Instance.Serialize();

                    success = true;
                }
                catch (Exception exception)
                {
                    ErrorDialog dialog = new ErrorDialog(exception);
                    dialog.Caption += " (Execute)";
                    dialog.ShowDialog();
                }
            }
            return success;
        }

        public bool Cleanup()
        {
            bool success = false;

            if (this.mainForm != null)
            {
                try
                {
                    // Close logging instance!
                    Logging.Instance.Close();

                    this.mainForm.Dispose();
                    this.mainForm = null;

                    success = true;
                }
                catch (Exception exception)
                {
                    ErrorDialog dialog = new ErrorDialog(exception);
                    dialog.Caption += " (Cleanup)";
                    dialog.ShowDialog();
                }
            }
            return success;
        }

        public MainForm MainForm
        {
            get { return this.mainForm; }
        }

        private Point LocationToScreen(Rectangle bounds)
        {
            foreach (Screen screen in Screen.AllScreens)
            {
                if (screen.Bounds.Contains(bounds))
                {
                    return bounds.Location; // Location is still within bounds.
                }
            }
            return new Point(50, 50); // Location is out of bounds, return with default location.
        }

        private void Setup(CommandLineParser parser)
        {
            try
            {
                // Try attaching to console window.
                if (!ConsoleNativeAccess.AttachConsole(ConsoleNativeAccess.ATTACH_PARENT_PROCESS))
                {
                    // Allocate console window if attaching fails.
                    ConsoleNativeAccess.AllocConsole();
                }

                // Create needed 
                string project = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
                string folder = project;

                // Prepare folder name and path
                if (folder[folder.Length - 1] != Path.DirectorySeparatorChar)
                {
                    folder += Path.DirectorySeparatorChar;
                }

                if (!String.IsNullOrEmpty(parser.Folder))
                {
                    if (parser.Folder[parser.Folder.Length - 1] != Path.DirectorySeparatorChar)
                    {
                        folder += parser.Folder + Path.DirectorySeparatorChar;
                    }
                    else
                    {
                        folder += parser.Folder;
                    }
                }

                // Prepare project name and path.
                if (project[project.Length - 1] != Path.DirectorySeparatorChar)
                {
                    project += Path.DirectorySeparatorChar;
                }
                project += parser.Project;

                // Add files according to given search criteria
                try
                {
                    ProjectSettings projectSettings = new ProjectSettings();

                    // Load all DLLs
                    this.Message("\r\nTry to load all available DLL files...");
                    this.AddToProjectSettings(parser, projectSettings, folder, "*.dll");

                    // Load all EXEs
                    this.Message("Try to load all available EXE files...");
                    this.AddToProjectSettings(parser, projectSettings, folder, "*.exe");

                    // Further settings
                    projectSettings.Saved = DateTime.Now;
                    projectSettings.Author = "generated";

                    // Save current project to given file.
                    ProjectSerializer serializer = new ProjectSerializer(null);
                    serializer.Settings = projectSettings;
                    serializer.Serialize(project);
                }
                catch (Exception exception)
                {
                    ErrorDialog dialog = new ErrorDialog(exception);
                    dialog.ShowDialog();
                    return; // Fail and don't save application settings!
                }

                // Save application settings into default configuration.
                ApplicationSettings.Instance.AutoOpenLastProject = parser.Autoload;
                ApplicationSettings.Instance.LastProject = project;
                ApplicationSettings.Instance.LastFolderImport = Path.GetDirectoryName(project);
                ApplicationSettings.Instance.MRUProjectsAdd(project);

                if (parser.IsLogging)
                {
                    ApplicationSettings.Instance.Logging.Enabled = true;
                    ApplicationSettings.Instance.Logging.Filename = parser.Logfile;
                    ApplicationSettings.Instance.Logging.Overwrite = parser.Overwrite;
                }

                ApplicationSettings.Instance.Serialize();

                this.Message("Creating project [" + parser.Project + "] done.\r\n");
            }
            finally
            {
                ConsoleNativeAccess.FreeConsole();
            }
        }

        private void AddToProjectSettings(CommandLineParser parser, ProjectSettings projectSettings, string folder, string search)
        {
            string[] files = System.IO.Directory.GetFiles(folder, search);
            foreach (string file in files)
            {
                try
                {
                    FBlockInvoker fblock = new FBlockInvoker(file, null);
                    if (fblock.IsValid)
                    {
                        if (fblock.Initialize(null))
                        {
                            ProjectEntry entry = new ProjectEntry();

                            // Always save FBlock ID with FBlock name.
                            entry.Name = string.Format("{0} (0x{1:X4})", fblock.Name, fblock.FBlockID);
                            entry.Assembly = fblock.AssemblyName;
                            entry.ShowUI = !parser.Hide;
                            entry.Connected = parser.Connect;
                            entry.Selected = false;

                            projectSettings.ProjectEntrys.Add(entry);

                            this.Message("Entry [" + entry.Name + "] added to the project.");
                        }
                        fblock.Cleanup();
                    }
                }
                catch (MissingMethodException /*exception*/)
                {
                    // Found an unloadable DLL/EXE
                    this.Warning("Could not load file [" + Path.GetFileName(file) + "]!");
                }
            }
        }

        #region IReportSink Members

        public void Message(string description)
        {
            this.Message(description, null, null, null);
        }

        public void Message(string description, string source)
        {
            this.Message(description, source, null, null);
        }

        public void Message(string description, string source, string category)
        {
            this.Message(description, source, category, null);
        }

        public void Message(string description, object additional)
        {
            this.Message(description, null, null, additional);
        }

        public void Message(string description, string source, object additional)
        {
            this.Message(description, source, null, additional);
        }

        public void Message(string description, string source, string category, object additional)
        {
            ConsoleColor old = System.Console.ForegroundColor;
            System.Console.ForegroundColor = ConsoleColor.White;
            System.Console.WriteLine(this.PrepareOutput(description, source, category, additional));
            System.Console.ForegroundColor = old;
        }

        public void Warning(string description)
        {
            this.Warning(description, null, null, null);
        }

        public void Warning(string description, string source)
        {
            this.Warning(description, source, null, null);
        }

        public void Warning(string description, string source, string category)
        {
            this.Warning(description, source, category, null);
        }

        public void Warning(string description, object additional)
        {
            this.Warning(description, null, null, additional);
        }

        public void Warning(string description, string source, object additional)
        {
            this.Warning(description, source, null, additional);
        }

        public void Warning(string description, string source, string category, object additional)
        {
            ConsoleColor old = System.Console.ForegroundColor;
            System.Console.ForegroundColor = ConsoleColor.Yellow;
            System.Console.WriteLine(this.PrepareOutput(description, source, category, additional));
            System.Console.ForegroundColor = old;
        }

        public void Error(string description)
        {
            this.Error(description, null, null, null);
        }

        public void Error(string description, string source)
        {
            this.Error(description, source, null, null);
        }

        public void Error(string description, string source, string category)
        {
            this.Error(description, source, category, null);
        }

        public void Error(string description, object additional)
        {
            this.Error(description, null, null, additional);
        }

        public void Error(string description, string source, object additional)
        {
            this.Error(description, source, null, additional);
        }

        public void Error(string description, string source, string category, object additional)
        {
            ConsoleColor old = System.Console.ForegroundColor;
            System.Console.ForegroundColor = ConsoleColor.Red;
            System.Console.WriteLine(this.PrepareOutput(description, source, category, additional));
            System.Console.ForegroundColor = old;
        }

        public void Debug(string description)
        {
            this.Debug(description, null, null, null);
        }

        public void Debug(string description, string source)
        {
            this.Debug(description, source, null, null);
        }

        public void Debug(string description, string source, string category)
        {
            this.Debug(description, source, category, null);
        }

        public void Debug(string description, object additional)
        {
            this.Debug(description, null, null, additional);
        }

        public void Debug(string description, string source, object additional)
        {
            this.Debug(description, source, null, additional);
        }

        public void Debug(string description, string source, string category, object additional)
        {
            ConsoleColor old = System.Console.ForegroundColor;
            System.Console.ForegroundColor = ConsoleColor.Gray;
            System.Console.WriteLine(this.PrepareOutput(description, source, category, additional));
            System.Console.ForegroundColor = old;
        }

        private string PrepareOutput(string description, string source, string category, object additional)
        {
            string result = "";

            if (!String.IsNullOrEmpty(description))
            {
                result += description;
            }

            if (!String.IsNullOrEmpty(source))
            {
                result += " [" + source + "]";
            }

            if (!String.IsNullOrEmpty(category))
            {
                result += " [" + category + "]";
            }

            if (additional != null)
            {
                result += "\r\n" + additional.ToString();
            }

            return result;
        }

        #endregion // IReportSink Members
    }

    internal static class ConsoleNativeAccess
    {
        internal static int ATTACH_PARENT_PROCESS = -1;

        [DllImport("kernel32.dll", SetLastError = true)]
        internal static extern bool AllocConsole();

        [DllImport("kernel32.dll", SetLastError = true)]
        internal static extern bool FreeConsole();

        [DllImport("kernel32.dll", SetLastError = true)]
        internal static extern bool AttachConsole(int idProcess);
    }
}

// EOF