﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections.Generic;

namespace fblock.manager
{
    public partial class ReportSink : TableLayoutPanel, IReportSink
    {
        private enum Nature
        {
            MESSAGE = 1,
            WARNING = 2,
            ERROR = 3,
            DEBUG = 4,
        }

        private List<ListViewItem> reportItems = new List<ListViewItem>();
        private List<ListViewItem> reportCache = new List<ListViewItem>();

        private string errorButtonLabel = "";
        private string warningButtonLabel = "";
        private string messageButtonLabel = "";

        public ReportSink()
            : base()
        {
            this.InitializeComponent();
            this.PostInitializeComponent();
        }

        public new ContextMenuStrip ContextMenuStrip
        {
            get { return this.reportListView.ContextMenuStrip; }
        }

        public ListView ReportListView
        {
            get { return this.reportListView; }
        }

        public void ApplicationSettingsChanged()
        {
            // Fit new limits.
            if (ApplicationSettings.Instance.ReportMaxEntries == 0)
            {
                // Zero limit means unlimited!
                this.FitMessageLimits(0);
            }
            else
            {
                // Leave the exact number of messages!
                this.FitMessageLimits(ApplicationSettings.Instance.ReportMaxEntries + 1);
            }

            // Re-filter items.
            this.FilterListItems();

            // Force updating the UI.
            Program.Instance.MainForm.Redraw();
        }

        #region IReportSink message function implementation

        public void Message(string description)
        {
            this.Message(description, null, null, null);
        }

        public void Message(string description, string source)
        {
            this.Message(description, source, null, null);
        }

        public void Message(string description, string source, string category)
        {
            this.Message(description, source, category, null);
        }

        public void Message(string description, object additional)
        {
            this.Message(description, null, null, additional);
        }

        public void Message(string description, string source, object additional)
        {
            this.Message(description, source, null, additional);
        }

        public void Message(string description, string source, string category, object additional)
        {
            try
            {
                this.AppendList(Nature.MESSAGE, description, source, category, additional);
            }
            catch (Exception /*exception*/)
            {
            }
        }

        #endregion

        #region IReportSink warning function implementation

        public void Warning(string description)
        {
            this.Warning(description, null, null, null);
        }

        public void Warning(string description, string source)
        {
            this.Warning(description, source, null, null);
        }

        public void Warning(string description, string source, string category)
        {
            this.Warning(description, source, category, null);
        }

        public void Warning(string description, object additional)
        {
            this.Warning(description, null, null, additional);
        }

        public void Warning(string description, string source, object additional)
        {
            this.Warning(description, source, null, additional);
        }

        public void Warning(string description, string source, string category, object additional)
        {
            try
            {
                this.AppendList(Nature.WARNING, description, source, category, additional);
            }
            catch (Exception /*exception*/)
            {
            }
        }

        #endregion

        #region IReportSink error function implementation

        public void Error(string description)
        {
            this.Error(description, null, null, null);
        }

        public void Error(string description, string source)
        {
            this.Error(description, source, null, null);
        }

        public void Error(string description, string source, string category)
        {
            this.Error(description, source, category, null);
        }

        public void Error(string description, object additional)
        {
            this.Error(description, null, null, additional);
        }

        public void Error(string description, string source, object additional)
        {
            this.Error(description, source, null, additional);
        }

        public void Error(string description, string source, string category, object additional)
        {
            try
            {
                this.AppendList(Nature.ERROR, description, source, category, additional);
            }
            catch (Exception /*exception*/)
            {
            }
        }

        #endregion

        #region IReportSink debug function implementation

        public void Debug(string description)
        {
#if DEBUG
            this.Debug(description, null, null, null);
#endif
        }

        public void Debug(string description, string source)
        {
#if DEBUG
            this.Debug(description, source, null, null);
#endif
        }

        public void Debug(string description, string source, string category)
        {
#if DEBUG
            this.Debug(description, source, category, null);
#endif
        }

        public void Debug(string description, object additional)
        {
#if DEBUG
            this.Debug(description, null, null, additional);
#endif
        }

        public void Debug(string description, string source, object additional)
        {
#if DEBUG
            this.Debug(description, source, null, additional);
#endif
        }

        public void Debug(string description, string source, string category, object additional)
        {
#if DEBUG
            try
            {
                this.AppendList(Nature.DEBUG, description, source, category, additional);
            }
            catch (Exception /*exception*/)
            {
            }
#endif
        }

        #endregion

        #region Context menu and item event handlers

        private void ContextMenu_Opening(object sender, CancelEventArgs args)
        {
            // Enable/disable menus item depending on current list item count.
            this.menuItemClear.Enabled = (this.reportItems.Count != 0);
            this.menuItemExport.Enabled = (this.reportItems.Count != 0);
        }

        private void MenuItemClear_Click(object sender, EventArgs args)
        {
            // Remove all existing combo box items.
            this.cmbFilterSource.Items.Clear();
            this.cmbFilterSource.Items.Add("");
            this.cmbFilterSource.SelectedIndex = 0;

            this.cmbFilterCategory.Items.Clear();
            this.cmbFilterCategory.Items.Add("");
            this.cmbFilterCategory.SelectedIndex = 0;

            // Remove all existing list box items and re-filter items.
            this.reportItems.Clear();
            this.FilterListItems();
        }

        private void MenuItemExport_Click(object sender, EventArgs args)
        {
            // Save all available list box items into an output file.
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "Protocol files (*.txt,*.log)|*.txt;*.log|All files (*.*)|*.*";
            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                FileStream stream = null;
                StreamWriter output = null;
                try
                {
                    stream = File.Create(dialog.FileName);
                    output = new StreamWriter(stream);
                    foreach (ListViewItem item in this.reportItems)
                    {
                        string line = "";

                        // Add time stamp
                        line += item.SubItems[1].Text;

                        // Add message type
                        line += String.Format(" {0,-9} ", ("[" + item.Tag.ToString()) + "]");

                        // Add source (if exist)
                        if (!String.IsNullOrEmpty(item.SubItems[2].Text))
                        {
                            line += item.SubItems[2].Text + ", ";
                        }

                        // Add category (if exist)
                        if (!String.IsNullOrEmpty(item.SubItems[3].Text))
                        {
                            line += item.SubItems[3].Text + ", ";
                        }

                        // Add description (if exist)
                        if (!String.IsNullOrEmpty(item.SubItems[4].Text))
                        {
                            line += item.SubItems[4].Text;
                        }

                        // Add additional (if exist)
                        if (!String.IsNullOrEmpty(item.SubItems[5].Text))
                        {
                            line += " (" + item.SubItems[5].Text + ")";
                        }

                        // Write current line to file
                        output.WriteLine(line);
                    }
                    output.Flush();
                }
                catch (Exception /*exception*/)
                {
                }
                finally
                {
                    if (output != null) { output.Close(); }
                    if (stream != null) { stream.Close(); }
                }
            }
            Program.Instance.MainForm.Redraw();
        }

        private void MenuItemAutoscroll_Click(object sender, EventArgs args)
        {
            this.menuItemAutoscroll.Checked = !this.menuItemAutoscroll.Checked;
            ApplicationSettings.Instance.AutoScroll = this.menuItemAutoscroll.Checked;
            this.UpdateAutoscrollButton();

            // Select last list box item if autoscroll is enabled.
            int last = this.reportListView.Items.Count - 1;
            if (last >= 0 && this.menuItemAutoscroll.Checked)
            {
                //this.reportListView.Items[last].Selected = true;
                this.reportListView.Items[last].EnsureVisible();
            }
        }

        #endregion

        #region Report toolbar event definition section

        private void FilterError_Click(object sender, EventArgs args)
        {
            this.btnShowErrors.Checked = !this.btnShowErrors.Checked;
            ApplicationSettings.Instance.ReportShowErrors = this.btnShowErrors.Checked;
            this.FilterListItems();
            this.UpdateShowErrorsButton();
        }

        private void FilterWarning_Click(object sender, EventArgs args)
        {
            this.btnShowWarnings.Checked = !this.btnShowWarnings.Checked;
            ApplicationSettings.Instance.ReportShowWarnings = this.btnShowWarnings.Checked;
            this.FilterListItems();
            this.UpdateShowWarningsButton();
        }

        private void FilterMessage_Click(object sender, EventArgs args)
        {
            this.btnShowMessages.Checked = !this.btnShowMessages.Checked;
            ApplicationSettings.Instance.ReportShowMessages = this.btnShowMessages.Checked;
            this.FilterListItems();
            this.UpdateShowMessagesButton();
        }

        private void FilterSource_SelectedIndexChanged(object sender, EventArgs args)
        {
            this.FilterListItems();
        }

        private void FilterCategory_SelectedIndexChanged(object sender, EventArgs args)
        {
            this.FilterListItems();
        }

        private void ButtonClear_Click(object sender, EventArgs args)
        {
            this.MenuItemClear_Click(this.btnClear, new EventArgs());
        }

        private void ButtonExport_Click(object sender, EventArgs args)
        {
            this.MenuItemExport_Click(this.btnExport, new EventArgs());
        }

        private void ButtonAutoscroll_Click(object sender, EventArgs args)
        {
            this.MenuItemAutoscroll_Click(this.btnAutoscroll, new EventArgs());
        }

        private void ButtonLogging_Click(object sender, EventArgs args)
        {
            LoggingDialog dialog = new LoggingDialog(ApplicationSettings.Instance.Logging);
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                // Take over made changes!
                ApplicationSettings.Instance.Logging.Copy(dialog.Settings);
            }
        }

        #endregion

        #region Report list view helper functions

        private delegate void AppendListDelegate(Nature nature, string description, string source, string category, object additional);
        private void AppendList(Nature nature, string description, string source, string category, object additional)
        {
            if (Program.Instance.MainForm.InvokeRequired)
            {
                Program.Instance.MainForm.Invoke(new AppendListDelegate(AppendList), nature, description, source, category, additional);
            }
            else
            {
                // Remove messages and keep one entry left for the new message!
                this.FitMessageLimits(ApplicationSettings.Instance.ReportMaxEntries);

                ListViewItem item = this.BuildOutput(nature, description, source, category, additional);
                this.reportItems.Add(item);

                this.FilterListItems();

                this.AddToCombobox(this.cmbFilterSource, source);
                this.AddToCombobox(this.cmbFilterCategory, category);

                // Force updating the UI.
                Program.Instance.MainForm.Redraw();

                // Write given message into log-file!
                this.WriteLogMessage(nature, description, source, category, additional);
            }
        }

        private ListViewItem BuildOutput(Nature nature, string description, string source, string category, object additional)
        {
            ListViewItem item = new ListViewItem();
            switch (nature)
            {
                case Nature.MESSAGE:
                    item.ImageIndex = this.reportListView.SmallImageList.Images.IndexOfKey(Nature.MESSAGE.ToString());
                    item.Tag = Nature.MESSAGE;
                    break;
                case Nature.WARNING:
                    item.ImageIndex = this.reportListView.SmallImageList.Images.IndexOfKey(Nature.WARNING.ToString());
                    item.Tag = Nature.WARNING;
                    break;
                case Nature.ERROR:
                    item.ImageIndex = this.reportListView.SmallImageList.Images.IndexOfKey(Nature.ERROR.ToString());
                    item.Tag = Nature.ERROR;
                    break;
                case Nature.DEBUG:
                    item.ImageIndex = this.reportListView.SmallImageList.Images.IndexOfKey(Nature.DEBUG.ToString());
                    item.Tag = Nature.DEBUG;
                    break;
            }

            item.SubItems.Add(DateTime.Now.ToString());

            if (!String.IsNullOrEmpty(source))
            {
                item.SubItems.Add(source);
            }
            else
            {
                item.SubItems.Add("");
            }

            if (!String.IsNullOrEmpty(category))
            {
                item.SubItems.Add(category);
            }
            else
            {
                item.SubItems.Add("");
            }

            if (!String.IsNullOrEmpty(description))
            {
                item.SubItems.Add(description);
            }
            else
            {
                item.SubItems.Add("");
            }

            if (additional != null)
            {
                item.SubItems.Add(additional.ToString());
            }
            else
            {
                item.SubItems.Add("");
            }

            return item;
        }

        private void WriteLogMessage(Nature nature, string description, string source, string category, object additional)
        {
            string output = "";

            if (!String.IsNullOrEmpty(description))
            {
                output += description;
            }

            if (!String.IsNullOrEmpty(source))
            {
                output += ((output.Length == 0) ? "" : " ") + "(" + source + ")";
            }

            if (!String.IsNullOrEmpty(category))
            {
                output += ((output.Length == 0) ? "" : " ") + "(" + category + ")";
            }

            if (additional != null)
            {
                output += ((output.Length == 0) ? "" : " => ") + additional.ToString();
            }

            switch (nature)
            {
                case Nature.MESSAGE:
                    Logging.Instance.Message(output);
                    break;
                case Nature.WARNING:
                    Logging.Instance.Warning(output);
                    break;
                case Nature.ERROR:
                    Logging.Instance.Error(output);
                    break;
                case Nature.DEBUG:
                    Logging.Instance.Debug(output);
                    break;
            }
        }

        private void AddToCombobox(ToolStripComboBox combo, string data)
        {
            foreach (object entry in combo.Items)
            {
                if (entry.ToString().Equals(data))
                {
                    return;
                }
            }
            combo.Items.Add(data);
        }

        private void AddToCache(ListViewItem item, bool check, string source, string category)
        {
            if (check)
            {
                if (String.IsNullOrEmpty(source) && String.IsNullOrEmpty(category))
                {
                    this.reportCache.Add(item);
                }
                else if (!String.IsNullOrEmpty(source) && String.IsNullOrEmpty(category))
                {
                    if (source.Equals(item.SubItems[2].Text))
                    {
                        this.reportCache.Add(item);
                    }
                }
                else if (String.IsNullOrEmpty(source) && !String.IsNullOrEmpty(category))
                {
                    if (category.Equals(item.SubItems[3].Text))
                    {
                        this.reportCache.Add(item);
                    }
                }
                else
                {
                    if (source.Equals(item.SubItems[2].Text) && category.Equals(item.SubItems[3].Text))
                    {
                        this.reportCache.Add(item);
                    }
                }
            }
        }

        private void FilterListItems()
        {
            // Disable list view drawing.
            this.reportListView.BeginUpdate();

            // Clean list view item cache.
            this.reportCache.Clear();

            string source = "";
            if (this.cmbFilterSource.Items.Count > 0)
            {
                source = this.cmbFilterSource.SelectedItem.ToString();
            }

            string category = "";
            if (this.cmbFilterCategory.Items.Count > 0)
            {
                category = this.cmbFilterCategory.SelectedItem.ToString();
            }

            int errors = 0;
            int warnings = 0;
            int messages = 0;

            foreach (ListViewItem item in this.reportItems)
            {
                switch ((Nature)item.Tag)
                {
                    case Nature.ERROR:
                        errors++;
                        this.AddToCache(item, this.btnShowErrors.Checked, source, category);
                        break;

                    case Nature.WARNING:
                        warnings++;
                        this.AddToCache(item, this.btnShowWarnings.Checked, source, category);
                        break;

                    case Nature.MESSAGE:
                        messages++;
                        this.AddToCache(item, this.btnShowMessages.Checked, source, category);
                        break;

                    case Nature.DEBUG:
                        this.AddToCache(item, true, "", "");
                        break;

                }
            }

            // Set list view's new virtual size.
            this.reportListView.VirtualListSize = this.reportCache.Count;

            // Select last list box item if autoscroll is enabled.
            int last = this.reportListView.Items.Count - 1;
            if (last >= 0 && this.menuItemAutoscroll.Checked)
            {
                //this.reportListView.Items[last].Selected = true;
                this.reportListView.Items[last].EnsureVisible();
            }

            // Enable list view drawing.
            this.reportListView.EndUpdate();

            this.btnShowErrors.Text = String.Format("{0} {1}", errors, this.errorButtonLabel);
            this.btnShowWarnings.Text = String.Format("{0} {1}", warnings, this.warningButtonLabel);
            this.btnShowMessages.Text = String.Format("{0} {1}", messages, this.messageButtonLabel);

            this.btnClear.Enabled = (this.reportItems.Count != 0);
            this.btnExport.Enabled = (this.reportItems.Count != 0);
        }

        private void FitMessageLimits(ulong limit)
        {
            try
            {
                // Zero limit means unlimited!
                if (limit != 0 && this.reportItems.Count >= (int)limit)
                {
                    while (this.reportItems.Count >= (int)limit)
                    {
                        this.reportItems.RemoveAt(0);
                    }
                }
            }
            catch (Exception exception)
            {
                System.Diagnostics.Debug.WriteLine(exception.ToString());
            }
        }

        private void ReportListView_RetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs args)
        {
            if (args.ItemIndex < this.reportCache.Count)
            {
                args.Item = this.reportCache[args.ItemIndex]; ;
            }
        }

        #endregion

        private void PostInitializeComponent()
        {
            this.errorButtonLabel = this.btnShowErrors.Text;
            this.warningButtonLabel = this.btnShowWarnings.Text;
            this.messageButtonLabel = this.btnShowMessages.Text;

            this.reportListView.SmallImageList = new ImageList();
            this.reportListView.SmallImageList.Images.Add(Nature.MESSAGE.ToString(), global::fblock.manager.Properties.Resources.Message);
            this.reportListView.SmallImageList.Images.Add(Nature.WARNING.ToString(), global::fblock.manager.Properties.Resources.Warning);
            this.reportListView.SmallImageList.Images.Add(Nature.ERROR.ToString(), global::fblock.manager.Properties.Resources.Error);
            this.reportListView.SmallImageList.Images.Add(Nature.DEBUG.ToString(), global::fblock.manager.Properties.Resources.Debug);

            this.reportListView.Columns.Add("", 22);
            this.reportListView.Columns.Add("Timestamp", 120);
            this.reportListView.Columns.Add("Source", 60);
            this.reportListView.Columns.Add("Category", 60);
            this.reportListView.Columns.Add("Description", 300);
            this.reportListView.Columns.Add("Additional", 70);

            // TODO: Make first column fixed sized!
            // Read this: http://www.codeproject.com/KB/list/myListViewNoSize.aspx
            // Nice but it locks all columns!

            this.contextMenu.RenderMode = ToolStripRenderMode.System;
            this.contextMenu.Renderer = new ToolStripProfessionalRenderer(new CustomColorTable());

            this.reportToolbar.RenderMode = ToolStripRenderMode.System;
            this.reportToolbar.Renderer = new ToolStripProfessionalRenderer(new CustomColorTable());

            this.cmbFilterSource.Items.Add( "" );
            this.cmbFilterSource.SelectedIndex = 0;

            this.cmbFilterCategory.Items.Add( "" );
            this.cmbFilterCategory.SelectedIndex = 0;

            this.btnShowErrors.Checked = ApplicationSettings.Instance.ReportShowErrors;
            this.btnShowWarnings.Checked = ApplicationSettings.Instance.ReportShowWarnings;
            this.btnShowMessages.Checked = ApplicationSettings.Instance.ReportShowMessages;
            this.menuItemAutoscroll.Checked = ApplicationSettings.Instance.AutoScroll;

            this.reportListView.VirtualMode = true;
            this.reportListView.VirtualListSize = 0;
            this.reportListView.RetrieveVirtualItem += new RetrieveVirtualItemEventHandler(ReportListView_RetrieveVirtualItem);

            PropertyInfo property = typeof(ListView).GetProperty("DoubleBuffered", BindingFlags.NonPublic | BindingFlags.Instance);
            property.SetValue(this.reportListView, true, null);

            this.UpdateAutoscrollButton();
            this.UpdateShowErrorsButton();
            this.UpdateShowWarningsButton();
            this.UpdateShowMessagesButton();
        }

        private void UpdateAutoscrollButton()
        {
            if (this.menuItemAutoscroll.Checked)
            {
                this.btnAutoscroll.Image = global::fblock.manager.Properties.Resources.ScrollOn;
                this.btnAutoscroll.ToolTipText = "Disable auto scroll";
            }
            else
            {
                this.btnAutoscroll.Image = global::fblock.manager.Properties.Resources.ScrollOff;
                this.btnAutoscroll.ToolTipText = "Enable auto scroll";
            }
        }

        private void UpdateShowErrorsButton()
        {
            if (this.btnShowErrors.Checked)
            {
                this.btnShowErrors.ToolTipText = "Hide errors";
            }
            else
            {
                this.btnShowErrors.ToolTipText = "Show errors";
            }
        }

        private void UpdateShowWarningsButton()
        {
            if (this.btnShowWarnings.Checked)
            {
                this.btnShowWarnings.ToolTipText = "Hide warnings";
            }
            else
            {
                this.btnShowWarnings.ToolTipText = "Show warnings";
            }
        }

        private void UpdateShowMessagesButton()
        {
            if (this.btnShowMessages.Checked)
            {
                this.btnShowMessages.ToolTipText = "Hide messages";
            }
            else
            {
                this.btnShowMessages.ToolTipText = "Show messages";
            }
        }
    }
}

// EOF