/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Windows.Forms;

// The usage of namespace IWshRuntimeLibrary requires an inclusion of the "Windows 
// Script Host Object Model" type library! Adding this COM reference will result in 
// the creation of file "Interop.IWshRuntimeLibrary.dll" during the build process. 
// This means, do not remove that output file and ship it when publishing!
using IWshRuntimeLibrary;

namespace fblock.manager
{
    public static class ShellLink
    {
        private static string EXTENSION = ".lnk";
        private static ReportSink reporter = null;

        public static void SetReportSink(ReportSink reporter)
        {
            ShellLink.reporter = reporter;
        }

        public static bool Exists(Environment.SpecialFolder folder, string name)
        {
            return ShellLink.Exists(Environment.GetFolderPath(folder), name);
        }

        public static bool Exists(string folder, string name)
        {
            // Some debug checks to inform users about invalid parameters.
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(folder));
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(name));

            // Determine existence of given folder.
            DirectoryInfo special = new DirectoryInfo(folder);
            FileInfo original = new FileInfo(name);
            string filename = special.FullName + Path.DirectorySeparatorChar + original.Name + ShellLink.EXTENSION;
            FileInfo file = new FileInfo(filename);
            return file.Exists;
        }

        public static bool Update(Environment.SpecialFolder folder, string target, string name, bool create)
        {
            return ShellLink.Update(Environment.GetFolderPath(folder), target, name, create);
        }

        public static bool Update(string folder, string target, string name, bool create)
        {
            // Some debug checks to inform users about invalid parameters.
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(folder));
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(target));
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(name));

            // Add or remove given target link.
            DirectoryInfo special = new DirectoryInfo(folder);
            FileInfo original = new FileInfo(name);
            string filename = special.FullName + Path.DirectorySeparatorChar + original.Name + ShellLink.EXTENSION;
            FileInfo file = new FileInfo(filename);

            if (create && !file.Exists)
            {
                try
                {
                    WshShell shell = new WshShell();
                    IWshShortcut link = (IWshShortcut)shell.CreateShortcut(file.FullName);
                    link.TargetPath = target;
                    link.Save();
                }
                catch (Exception exception)
                {
                    if (ShellLink.reporter != null)
                    {
                        ShellLink.reporter.Warning("Could not create shortcut \"" + name + "\"!", "FBlockManager", "Shell", exception);
                    }
                    return false;
                }
            }
            else if (!create && file.Exists)
            {
                try
                {
                    file.Delete();
                }
                catch (Exception exception)
                {
                    if (ShellLink.reporter != null)
                    {
                        ShellLink.reporter.Warning("Could not remove shortcut \"" + name + "\"!", "FBlockManager", "Shell", exception);
                    }
                    return false;
                }
            }
            return true;
        }

        public static bool IsOnDesktop(string name)
        {
            return ShellLink.Exists(Environment.SpecialFolder.DesktopDirectory, name);
        }

        public static bool AddDesktop(string name, string target)
        {
            return ShellLink.Update(Environment.SpecialFolder.DesktopDirectory, target, name, true);
        }

        public static bool DelDesktop(string name, string target)
        {
            return ShellLink.Update(Environment.SpecialFolder.DesktopDirectory, target, name, false);
        }

        public static bool IsOnSendTo(string name)
        {
            return ShellLink.Exists(Environment.SpecialFolder.SendTo, name);
        }

        public static bool AddSendTo(string name, string target)
        {
            return ShellLink.Update(Environment.SpecialFolder.SendTo, target, name, true);
        }

        public static bool DelSendTo(string name, string target)
        {
            return ShellLink.Update(Environment.SpecialFolder.SendTo, target, name, false);
        }

        public static bool IsOnPrograms(string name)
        {
            return ShellLink.IsOnPrograms(name, null);
        }

        public static bool AddPrograms(string name, string target)
        {
            return ShellLink.AddPrograms(name, null, target);
        }

        public static bool DelPrograms(string name, string target)
        {
            return ShellLink.DelPrograms(name, null, target);
        }

        public static bool IsOnPrograms(string name, string group)
        {
            if (!String.IsNullOrEmpty(group))
            {
                DirectoryInfo programs = new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.Programs));
                string helper = programs.FullName + Path.DirectorySeparatorChar + group;
                return ShellLink.Exists(helper, name);
            }
            else
            {
                return ShellLink.Exists(Environment.SpecialFolder.Programs, name);
            }
        }

        public static bool AddPrograms(string name, string group, string target)
        {
            if (!String.IsNullOrEmpty(group))
            {
                try
                {
                    DirectoryInfo programs = new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.Programs));
                    string destination = programs.FullName + Path.DirectorySeparatorChar + group;

                    // Create program group beforehand!
                    Directory.CreateDirectory(destination);

                    return ShellLink.Update(destination, target, name, true);

                }
                catch (Exception /*exception*/)
                {
                    // TODO: Enable Report Sink access!
                    return false;
                }
            }
            else
            {
                return ShellLink.Update(Environment.SpecialFolder.Programs, target, name, true);
            }
        }

        public static bool DelPrograms(string name, string group, string target)
        {
            if (!String.IsNullOrEmpty(group))
            {
                try
                {
                    DirectoryInfo programs = new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.Programs));
                    string destination = programs.FullName + Path.DirectorySeparatorChar + group;

                    if (ShellLink.Update(destination, target, name, false))
                    {
                        // Remove program group afterwards!
                        Directory.Delete(destination);

                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                catch (Exception /*exception*/)
                {
                    // TODO: Enable Report Sink access!
                    return false;
                }
            }
            else
            {
                return ShellLink.Update(Environment.SpecialFolder.Programs, target, name, false);
            }
        }
    }
}

// EOF